@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24885 $
+ * $Revision: 26053 $
  * $Author: rjongbloed $
- * $Date: 2010-11-21 17:31:12 -0600 (Sun, 21 Nov 2010) $
+ * $Date: 2011-06-20 03:40:20 -0500 (Mon, 20 Jun 2011) $
  */
 
 #ifndef OPAL_CODEC_OPALPLUGIN_H
@@ -36,6 +36,14 @@
 extern "C" {
 #endif
 
+#ifndef _CRT_NONSTDC_NO_DEPRECATE
+#define _CRT_NONSTDC_NO_DEPRECATE 1
+#endif
+
+#ifndef _CRT_SECURE_NO_WARNINGS
+#define _CRT_SECURE_NO_WARNINGS 1
+#endif
+
 #include <time.h>
 
 #ifdef _MSC_VER
@@ -49,6 +57,10 @@ extern "C" {
 #    define PLUGIN_CODEC_DLL_API __declspec(dllimport)
 #  endif
 
+#if !defined(strcasecmp) && !defined(_WIN32_WCE)
+#define strcasecmp stricmp
+#endif
+
 #else
 
 #define PLUGIN_CODEC_DLL_API
@@ -164,6 +176,9 @@ enum PluginCodec_Flags {
 
   PluginCodec_BitsPerSamplePos       = 12,
   PluginCodec_BitsPerSampleMask      = 0xf000,
+
+  PluginCodec_ChannelsPos            = 16,
+  PluginCodec_ChannelsMask           = 0x003f0000
 };
 
 enum PluginCodec_CoderFlags {
@@ -271,8 +286,10 @@ struct PluginCodec_Option {
 // Normalised option names
 #define PLUGINCODEC_OPTION_NEEDS_JITTER               "Needs Jitter"
 #define PLUGINCODEC_OPTION_CLOCK_RATE                 "Clock Rate"
+#define PLUGINCODEC_OPTION_CHANNELS                   "Channels"
 #define PLUGINCODEC_OPTION_FRAME_TIME                 "Frame Time"
 #define PLUGINCODEC_OPTION_MAX_FRAME_SIZE             "Max Frame Size"
+#define PLUGINCODEC_OPTION_MAX_TX_PACKET_SIZE         "Max Tx Packet Size"
 #define PLUGINCODEC_OPTION_MAX_BIT_RATE               "Max Bit Rate"
 #define PLUGINCODEC_OPTION_TARGET_BIT_RATE            "Target Bit Rate"
 #define PLUGINCODEC_OPTION_RX_FRAMES_PER_PACKET       "Rx Frames Per Packet"
