@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23934 $
+ * $Revision: 26036 $
  * $Author: rjongbloed $
- * $Date: 2010-01-13 18:47:35 -0600 (Wed, 13 Jan 2010) $
+ * $Date: 2011-06-16 20:40:56 -0500 (Thu, 16 Jun 2011) $
  */
 
 #ifndef OPAL_CODEC_OPALPLUGINMGR_H
@@ -81,6 +81,8 @@ class OpalPluginCodecHandler : public PObject
   public:
     OpalPluginCodecHandler();
 
+    static int GetChannelCount(const PluginCodec_Definition * codeDefn);
+
     virtual OpalMediaFormatInternal * OnCreateAudioFormat(OpalPluginCodecManager & mgr,
                                             const PluginCodec_Definition * codecDefn,
                                                               const char * fmtName,
@@ -246,10 +248,10 @@ class OpalPluginAudioFormatInternal : public OpalAudioFormatInternal, public Opa
     OpalPluginAudioFormatInternal(
       const PluginCodec_Definition * codecDefn,
       const char * fmtName,
-      const char * rtpEncodingName, /// rtp encoding name
-      unsigned frameTime,           /// Time for frame in RTP units (if applicable)
-      unsigned /*timeUnits*/,       /// RTP units for frameTime (if applicable)
-      time_t timeStamp              /// timestamp (for versioning)
+      const char * rtpEncodingName,
+      unsigned frameTime,
+      unsigned timeUnits,
+      time_t timeStamp
     );
     virtual PObject * Clone() const;
     virtual bool IsValidForProtocol(const PString & protocol) const;
@@ -321,9 +323,13 @@ class OpalPluginVideoTranscoder : public OpalVideoTranscoder, public OpalPluginT
   protected:
     bool EncodeFrames(const RTP_DataFrame & src, RTP_DataFrameList & dstList);
     bool DecodeFrames(const RTP_DataFrame & src, RTP_DataFrameList & dstList);
+    bool DecodeFrame(const RTP_DataFrame & src, RTP_DataFrameList & dstList);
 
     RTP_DataFrame * m_bufferRTP;
-    PTimeInterval   m_lastVideoFastUpdate;
+    DWORD           m_lastDecodedTimestamp; // For missing marker bit detection
+    DWORD           m_lastMarkerTimestamp;  // For continuous marker bit detection
+    bool            m_badMarkers;
+    PSimpleTimer    m_videoPictureLossTimer;
 
 #if PTRACING
     unsigned m_consecutiveIntraFrames;
@@ -356,7 +362,7 @@ class OpalPluginFaxFormatInternal : public OpalMediaFormatInternal, public OpalP
 
 //////////////////////////////////////////////////////
 //
-//  this is the base class for codecs accesible via the abstract factory functions
+//  this is the base class for codecs accessible via the abstract factory functions
 //
 
 /**Class for codcs which is accessible via the abstract factory functions.
