@@ -24,9 +24,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 21293 $
- * $Author: rjongbloed $
- * $Date: 2008-10-12 18:24:41 -0500 (Sun, 12 Oct 2008) $
+ * $Revision: 24722 $
+ * $Author: dereksmithies $
+ * $Date: 2010-09-23 23:32:35 -0500 (Thu, 23 Sep 2010) $
  */
 
 #ifndef OPAL_IAX2_CALLPROCESSOR_H
@@ -106,7 +106,7 @@ class IAX2CallProcessor : public IAX2Processor
   */
   virtual PBoolean SetUpConnection();
 
-  /**Return PTrue if the remote info in the frame matches the remote info in
+  /**Return true if the remote info in the frame matches the remote info in
      this connection */
   PBoolean Matches(IAX2Frame *frame) { return remote == (frame->GetRemoteInfo()); }
   
@@ -118,7 +118,7 @@ class IAX2CallProcessor : public IAX2Processor
      in/out packets) to be printed*/
   void ReportStatistics();  
 
-  /**Return PTrue if the arg matches the source call number for this connection */
+  /**Return true if the arg matches the source call number for this connection */
   PBoolean MatchingLocalCallNumber(PINDEX compare) { return (compare == remote.SourceCallNumber()); }  
   
   /**Get the bit pattern of the selected codec */
@@ -171,6 +171,10 @@ class IAX2CallProcessor : public IAX2Processor
   /**Get the username*/
   PString GetUserName() const;
   
+  /**Report the calling name - which we send to the remote endpoint
+     when making a call */
+  PString GetCallingName() const { return callingName; }
+
   /**Set the password for when we connect to a remote node
      we use it as authentication.  Note this must only be
      used before the main thread is started.  This is optional
@@ -256,8 +260,9 @@ class IAX2CallProcessor : public IAX2Processor
   /**Internal method to process an incoming network frame of type
      IAX2FullFrameProtocol.
      
-  A frame of FullFrameProtocol type is labelled as AST_FRAME_IAX in the asterisk souces,
-  It will contain 0, 1, 2 or more Information Elements (Ie) in the data section.*/
+  A frame of FullFrameProtocol type is labelled as AST_FRAME_IAX in
+  the asterisk souces, It will contain 0, 1, 2 or more Information
+  Elements (Ie) in the data section.*/
   virtual PBoolean ProcessNetworkFrame(IAX2FullFrameProtocol * src);
   
   /**Internal method to process an incoming network frame of type
@@ -293,7 +298,7 @@ class IAX2CallProcessor : public IAX2Processor
      The data in the array is already compressed. */
   void SendSoundMessage(PBYTEArray *sound);
   
-  /**Sends a transfer message ONLY if doTransfer is PTrue*/
+  /**Sends a transfer message ONLY if doTransfer is true*/
   void SendTransferMessage();
   
   /**Send a message to put the remote connection on hold*/
@@ -435,13 +440,28 @@ class IAX2CallProcessor : public IAX2Processor
      Provision device  */
   void ProcessIaxCmdProvision(IAX2FullFrameProtocol *src);
   
-  /** Process a FullFrameProtocol class, where the sub Class value is Download
-     firmware  */
+  /** Process a FullFrameProtocol class, where the sub Class value is
+     Download firmware  */
   void ProcessIaxCmdFwDownl(IAX2FullFrameProtocol *src);
   
-  /** Process a FullFrameProtocol class, where the sub Class value is Firmware
-     Data  */
+  /** Process a FullFrameProtocol class, where the sub Class value is
+     Firmware Data  */
   void ProcessIaxCmdFwData(IAX2FullFrameProtocol *src);
+
+  /** Process a FullFrameProtocol class, where the sub Class value is
+     CallToken. This is an authentication like packet that IAX2 added
+     in sep 2009 to prevent a DOS attack from too many incoming
+     callers.  
+
+     Reply with a NEW packet and the calltoken from supplied frame.*/
+  void ProcessIaxCmdCallToken(IAX2FullFrameProtocol *src);
+
+  /**We have to send a new frame to the remote host. This builds the
+     frame, and adds many of the information elements required. does
+     not add the calltoken ie - that is added elsewhere. This method
+     assumes this class already holds the relevant information on the
+     destination box */
+  IAX2FullFrameProtocol *BuildNewFrameForSending(IAX2FullFrameProtocol *inReplyTo = NULL);
   
   /**Count of the number of sound frames sent */
   PAtomicInteger audioFramesSent;
@@ -527,35 +547,35 @@ class IAX2CallProcessor : public IAX2Processor
   unsigned short callStatus;
   
   /** Mark call status as having sent a iaxcmdRinging packet */
-  void SetCallSentRinging(PBoolean newValue = PTrue) 
+  void SetCallSentRinging(PBoolean newValue = true) 
     { if (newValue) callStatus |= callSentRinging; else callStatus &= ~callSentRinging; }
   
   /** Mark call status as having received a new packet */
-  void SetCallNewed(PBoolean newValue = PTrue) 
+  void SetCallNewed(PBoolean newValue = true) 
     { if (newValue) callStatus |= callNewed; else callStatus &= ~callNewed; }
   
   /** Mark call status Registered (argument determines flag status) */
-  void SetCallRegistered(PBoolean newValue = PTrue) 
+  void SetCallRegistered(PBoolean newValue = true) 
     { if (newValue) callStatus |= callRegistered; else callStatus &= ~callRegistered; }
   
   /** Mark call status Authorised (argument determines flag status) */
-  void SetCallAuthorised(PBoolean newValue = PTrue) 
+  void SetCallAuthorised(PBoolean newValue = true) 
     { if (newValue) callStatus |= callAuthorised; else callStatus &= ~callAuthorised; }
   
   /** Mark call status Accepted (argument determines flag status) */
-  void SetCallAccepted(PBoolean newValue = PTrue) 
+  void SetCallAccepted(PBoolean newValue = true) 
     { if (newValue) callStatus |= callAccepted; else callStatus &= ~callAccepted; }
   
   /** Mark call status Ringing (argument determines flag status) */
-  void SetCallRinging(PBoolean newValue = PTrue) 
+  void SetCallRinging(PBoolean newValue = true) 
     { if (newValue) callStatus |= callRinging; else callStatus &= ~callRinging; }
   
   /** Mark call status Answered (argument determines flag status) */
-  void SetCallAnswered(PBoolean newValue = PTrue) 
+  void SetCallAnswered(PBoolean newValue = true) 
     { if (newValue) callStatus |= callAnswered; else callStatus &= ~callAnswered; }
 
   /** Mark call status as terminated (is processing IAX2 hangup packets etc)*/
-  void SetCallTerminating(PBoolean newValue = PTrue) 
+  void SetCallTerminating(PBoolean newValue = true) 
     { if (newValue) callStatus |= callTerminating; else callStatus &= ~callTerminating; }
   
   /** See if any of the flag bits are on, which indicate this call is actually active */
@@ -655,6 +675,21 @@ class IAX2CallProcessor : public IAX2Processor
   /**Process a full frame and respond accordingly to it*/
   virtual void ProcessFullFrame(IAX2FullFrame & fullFrame);
   
+  /**The calling Name - used when we send a new packet to the remote
+     host - this name is put the new packet */
+  PString callingName;
+
+  /**The calling extension - derived from the destination of the
+     call */
+  PString callingExtension;
+
+  /**The Dnid - used in creating a cmdNew packet for creating call */
+  PString callingDnid;
+
+  /**the context index - used in creating a cmdNew packet for creating
+     call */
+  PString callingContext;
+
   /**Optional username for when we connect to a remote node
      we use it as authentication.  Note this must only be
      set before the main thread is started.*/
@@ -676,14 +711,19 @@ class IAX2CallProcessor : public IAX2Processor
   
   /**The context to call for a transfer -- this is optional*/
   PString transferCalledContext;    
+
+  /**Flag to indicate we should not send the hangup frame on call
+     termination.  Normally, when a call ends, we send a hangup
+     frame. However, in the case where the remote endpoint rejects our
+     call, we are not supposed to send a hangup frame */
+  PBoolean suppressHangupFrame;
 };
 
 /////////////////////////////////////////////////////////////////////////
 
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 4 spaces.     */
-
-/*
+/* The comment below is magic for those who use emacs to edit this file.
+ * With the comment below, the tab key does auto indent to 2 spaces.    
+ *
  * Local Variables:
  * mode:c
  * c-basic-offset:2
