@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24442 $
+ * $Revision: 26231 $
  * $Author: rjongbloed $
- * $Date: 2010-06-02 03:17:31 -0500 (Wed, 02 Jun 2010) $
+ * $Date: 2011-07-19 21:23:34 -0500 (Tue, 19 Jul 2011) $
  */
 
 #ifndef OPAL_OPAL_ENDPOINT_H
@@ -70,7 +70,7 @@ class OpalEndPoint : public PObject
   public:
     enum Attributes {
       CanTerminateCall = 1,
-      HasLineInterface = 2
+      SupportsE164 = 2
     };
 
   /**@name Construction */
@@ -110,6 +110,12 @@ class OpalEndPoint : public PObject
     /**Add a set of listeners to the endoint.
        This allows for the automatic creating of incoming call connections. 
        If the list is empty then GetDefaultListeners() is used.
+
+       Note: while the \p interfaces parameter is a string array, each element
+       of the array should be compatible with OpalTransportAddress.
+
+       See OpalTransportAddress for more details on the syntax of an interface
+       definition.
       */
     PBoolean StartListeners(
       const PStringArray & interfaces ///<  Address of interface to listen on.
@@ -118,6 +124,9 @@ class OpalEndPoint : public PObject
     /**Add a listener to the endoint.
        This allows for the automatic creating of incoming call connections. /
        If the address is empty then the first entry of GetDefaultListeners() is used.
+
+       See OpalTransportAddress for more details on the syntax of an interface
+       definition.
       */
     PBoolean StartListener(
       const OpalTransportAddress & iface ///<  Address of interface to listen on.
@@ -138,14 +147,26 @@ class OpalEndPoint : public PObject
       */
     virtual PStringArray GetDefaultListeners() const;
 
+    /**Get comma separated list of transport protocols to create if
+       no explicit listeners started.
+      */
+    virtual PString GetDefaultTransport() const;
+
     /**Find a listener given the transport address.
       */
     OpalListener * FindListener(
         const OpalTransportAddress & iface ///<  Address of interface we may be listening on.
     );
 
+    /** Find a listener that is compatible with the specified protocol
+     */
+    bool FindListenerForProtocol(
+      const char * proto,         ///< Protocol to findlistener, e.g "tcp" or "udp"
+      OpalTransportAddress & addr ///< Address of listner interface
+    );
+
     /**Stop a listener given the transport address.
-       Returns PTrue if a listener was on that interface, and was stopped.
+       Returns true if a listener was on that interface, and was stopped.
       */
     PBoolean StopListener(
         const OpalTransportAddress & iface ///<  Address of interface we may be listening on.
@@ -161,7 +182,7 @@ class OpalEndPoint : public PObject
     /**Return a list of the transport addresses for all listeners on this endpoint
       */
     OpalTransportAddressArray GetInterfaceAddresses(
-      PBoolean excludeLocalHost = PTrue,       ///<  Flag to exclude 127.0.0.1
+      PBoolean excludeLocalHost = true,       ///<  Flag to exclude 127.0.0.1
       const OpalTransport * associatedTransport = NULL
                           ///<  Associated transport for precedence and translation
     );
@@ -174,15 +195,25 @@ class OpalEndPoint : public PObject
 
     /**Handle new incoming connection from listener.
 
-       A return value of PTrue indicates that the transport object should be
-       deleted by the caller. PFalse indicates that something else (eg the
+       A return value of true indicates that the transport object should be
+       deleted by the caller. false indicates that something else (eg the
        connection) has taken over responsibility for deleting the transport.
 
-       The default behaviour just returns PTrue.
+       The default behaviour just returns true.
       */
     virtual PBoolean NewIncomingConnection(
       OpalTransport * transport  ///<  Transport connection came in on
     );
+
+    /**Call back for a new connection has been constructed.
+       This is called after CreateConnection has returned a new connection.
+       It allows an application to make any custom adjustments to the
+       connection before it begins to process the protocol. behind it.
+      */
+    virtual void OnNewConnection(
+      OpalCall & call,              ///< Call that owns the newly created connection.
+      OpalConnection & connection   ///< New connection just created
+    );
   //@}
 
   /**@name Connection management */
@@ -203,12 +234,12 @@ class OpalEndPoint : public PObject
 
        The proto field is optional when passed to a specific endpoint. If it
        is present, however, it must agree with the endpoints protocol name or
-       PFalse is returned.
+       false is returned.
 
        This function usually returns almost immediately with the connection
        continuing to occur in a new background thread.
 
-       If PFalse is returned then the connection could not be established. For
+       If false is returned then the connection could not be established. For
        example if a PSTN endpoint is used and the assiciated line is engaged
        then it may return immediately. Returning a non-NULL value does not
        mean that the connection will succeed, only that an attempt is being
@@ -220,8 +251,8 @@ class OpalEndPoint : public PObject
       OpalCall & call,          ///<  Owner of connection
       const PString & party,    ///<  Remote party to call
       void * userData = NULL,          ///<  Arbitrary data to pass to connection
-      unsigned int options = 0,     ///<  options to pass to conneciton
-      OpalConnection::StringOptions * stringOptions = NULL
+      unsigned int options = 0,     ///<  Options bit mask to pass to conneciton
+      OpalConnection::StringOptions * stringOptions = NULL ///< Options to pass to connection
     ) = 0;
 
     /**Callback for outgoing connection, it is invoked after OpalLineConnection::SetUpConnection
@@ -233,7 +264,7 @@ class OpalEndPoint : public PObject
        This function is used for an application to control the answering of
        incoming calls.
 
-       If PTrue is returned then the connection continues. If PFalse then the
+       If true is returned then the connection continues. If false then the
        connection is aborted.
 
        Note this function should not block for any length of time. If the
@@ -253,7 +284,7 @@ class OpalEndPoint : public PObject
     virtual PBoolean OnIncomingConnection(
       OpalConnection & connection,  ///<  Connection that is calling
       unsigned options,             ///<  options for new connection (can't use default value as overrides will fail)
-      OpalConnection::StringOptions * stringOptions
+      OpalConnection::StringOptions * stringOptions  ///< Options to pass to connection
     );
 
     /**Call back for remote party is now responsible for completing the call.
@@ -279,7 +310,7 @@ class OpalEndPoint : public PObject
        remote endpoint is "ringing". Generally some time after the
        MakeConnection() function was called, this is function is called.
 
-       If PFalse is returned the connection is aborted.
+       If false is returned the connection is aborted.
 
        If an application overrides this function, it should generally call the
        ancestor version for correct operation. An application would typically
@@ -387,6 +418,31 @@ class OpalEndPoint : public PObject
       const PString & remoteParty         ///<  The new remote party
     );
 
+    /**A call back function to monitor the progress of a transfer.
+       When a transfer operation is initiated, the Transfer() function will
+       generally return immediately and the transfer may take some time. This
+       call back can give indicateion to the application of the progress of
+       the transfer.
+
+       The exact format of the \p info parameter is dependent on the protocol
+       being used. It will always have a value info["result"] which will be
+       "success" or "failed". Other variables are protocol dependent.
+
+       For SIP, there is an additional info["state"] containing the NOTIFY
+       subscription state and an info["code"] entry containing the 3 digit
+       code returned in the NOTIFY body.
+
+       A return value of false will immediately disconnect the current call.
+
+       The default behaviour calls the OpalManager function of the same name.
+       The default action of that function is to return false, thereby
+       releasing the connection if the info["result"] == "success".
+      */
+    virtual bool OnTransferNotify(
+      OpalConnection & connection,  ///< Connection being transferred.
+      const PStringToString & info  ///< Information on the transfer
+    );
+
     /**Clear a call.
        This finds the call by using the token then calls the OpalCall::Clear()
        function on it. All connections are released, and the conenctions and
@@ -418,7 +474,7 @@ class OpalEndPoint : public PObject
       */
     virtual void ClearAllCalls(
       OpalConnection::CallEndReason reason = OpalConnection::EndedByLocalUser, ///<  Reason for call clearing
-      PBoolean wait = PTrue   ///<  Flag for wait for calls to e cleared.
+      PBoolean wait = true   ///<  Flag for wait for calls to e cleared.
     );
 
     /**Find a connection that uses the specified token.
@@ -427,7 +483,7 @@ class OpalEndPoint : public PObject
       */
     PSafePtr<OpalConnection> GetConnectionWithLock(
       const PString & token,     ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite
+      PSafetyMode mode = PSafeReadWrite ///< Locking mode
     ) { return connectionsActive.FindWithLock(token, mode); }
 
     /**Find a connection that uses the specified token.
@@ -439,7 +495,7 @@ class OpalEndPoint : public PObject
     template <class ConnClass>
     PSafePtr<ConnClass> GetConnectionWithLockAs(
       const PString & token,     ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite
+      PSafetyMode mode = PSafeReadWrite ///< Locking mode
     )
     {
       PSafePtr<ConnClass> connection = PSafePtrCast<OpalConnection, ConnClass>(GetConnectionWithLock(token, mode));
@@ -582,6 +638,32 @@ class OpalEndPoint : public PObject
     );
   //@}
 
+  /**@name Instant Messaging */
+  //@{
+    /**Send text message
+     */
+    virtual PBoolean Message(
+      const PString & to, 
+      const PString & body
+    );
+    virtual PBoolean Message(
+      const PURL & to, 
+      const PString & type,
+      const PString & body,
+      PURL & from, 
+      PString & conversationId
+    );
+    virtual PBoolean Message(
+      OpalIM & Message
+    );
+
+    /**Called when text message received
+     */
+    virtual void OnMessageReceived(
+      const OpalIM & message
+    );
+  //@}
+
   /**@name Other services */
   //@{
     /**Callback called when Message Waiting Indication (MWI) is received
@@ -593,7 +675,7 @@ class OpalEndPoint : public PObject
     );
 
     /** Execute garbage collection for endpoint.
-        Returns PTrue if all garbage has been collected.
+        Returns true if all garbage has been collected.
         Default behaviour deletes the objects in the connectionsActive list.
       */
     virtual PBoolean GarbageCollection();
@@ -678,59 +760,10 @@ class OpalEndPoint : public PObject
     void SetSendUserInputMode(OpalConnection::SendUserInputModes mode) { defaultSendUserInputMode = mode; }
   //@}
 
-    virtual PString GetDefaultTransport() const;
-
-    /**Call back for a new connection has been constructed.
-       This is called after CreateConnection has returned a new connection.
-       It allows an application to make any custom adjustments to the
-       connection before it begins to process the protocol. behind it.
-      */
-    virtual void OnNewConnection(
-      OpalCall & call,              ///< Call that owns the newly created connection.
-      OpalConnection & connection   ///< New connection just created
-    );
-
 #if OPAL_PTLIB_SSL
-    PString GetSSLCertificate() const;
-#endif
-
-    /** Find a listener that is compatible with the specified protocol
-     */
-    bool FindListenerForProtocol(const char * protoPrefix, OpalTransportAddress & addr);
-
-    /**Send text message
-     */
-    virtual PBoolean Message(
-      const PString & to, 
-      const PString & body
-    );
-    virtual PBoolean Message(
-      const PURL & to, 
-      const PString & type,
-      const PString & body,
-      PURL & from, 
-      PString & conversationId
-    );
-
-    /**Called when text message received
-     */
-    virtual void OnMessageReceived(
-      const PURL & from, 
-      const PString & fromName,
-      const PURL & to, 
-      const PString & type,
-      const PString & body,
-      const PString & conversationId
-    );
-
-#if OPAL_HAS_IM
-    /** Called when text message to be sent to remote end of a connection
+    /** Get the name of the file to use as SSL certificate for SSL based calls, e.g. sips or h323s
       */
-    virtual bool TransmitExternalIM(
-      OpalConnection & conn, 
-      const OpalMediaFormat & format, 
-      RTP_IMFrame & frame
-    );
+    PString GetSSLCertificate() const;
 #endif
 
   protected:
@@ -738,6 +771,7 @@ class OpalEndPoint : public PObject
     PCaselessString prefixName;
     unsigned        attributeBits;
     WORD            defaultSignalPort;
+    PINDEX          m_maxSizeUDP;
     OpalProductInfo productInfo;
     PString         defaultLocalPartyName;
     PString         defaultDisplayName;
@@ -763,6 +797,7 @@ class OpalEndPoint : public PObject
     P_REMOVE_VIRTUAL(PBoolean, OnIncomingConnection(OpalConnection &, unsigned), false);
     P_REMOVE_VIRTUAL(PBoolean, OnIncomingConnection(OpalConnection &), false);
     P_REMOVE_VIRTUAL_VOID(AdjustMediaFormats(const OpalConnection &, OpalMediaFormatList &) const);
+    P_REMOVE_VIRTUAL_VOID(OnMessageReceived(const PURL&,const PString&,const PURL&,const PString&,const PString&,const PString&));
 };
 
 
