@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24074 $
+ * $Revision: 26270 $
  * $Author: rjongbloed $
- * $Date: 2010-02-23 20:36:11 -0600 (Tue, 23 Feb 2010) $
+ * $Date: 2011-08-03 03:29:01 -0500 (Wed, 03 Aug 2011) $
  */
 
 #ifndef OPAL_SIP_SIPEP_H
@@ -93,7 +93,9 @@ class SIPEndPoint : public OpalRTPEndPoint
     /**Create a new endpoint.
      */
     SIPEndPoint(
-      OpalManager & manager
+      OpalManager & manager,
+      unsigned maxConnectionThreads = 10,
+      unsigned maxHandlerThreads = 5
     );
 
     /**Destroy endpoint.
@@ -138,12 +140,12 @@ class SIPEndPoint : public OpalRTPEndPoint
 
        The proto field is optional when passed to a specific endpoint. If it
        is present, however, it must agree with the endpoints protocol name or
-       PFalse is returned.
+       false is returned.
 
        This function usually returns almost immediately with the connection
        continuing to occur in a new background thread.
 
-       If PFalse is returned then the connection could not be established. For
+       If false is returned then the connection could not be established. For
        example if a PSTN endpoint is used and the assiciated line is engaged
        then it may return immediately. Returning a non-NULL value does not
        mean that the connection will succeed, only that an attempt is being
@@ -182,7 +184,7 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
 
     /** Execute garbage collection for endpoint.
-        Returns PTrue if all garbage has been collected.
+        Returns true if all garbage has been collected.
         Default behaviour deletes the objects in the connectionsActive list.
       */
     virtual PBoolean GarbageCollection();
@@ -215,7 +217,7 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
     
     /**Forward the connection using the same token as the specified connection.
-       Return PTrue if the connection is being redirected.
+       Return true if the connection is being redirected.
       */
     virtual PBoolean ForwardConnection(
       SIPConnection & connection,     ///<  Connection to be forwarded
@@ -347,8 +349,8 @@ class SIPEndPoint : public OpalRTPEndPoint
       */
     PSafePtr<SIPConnection> GetSIPConnectionWithLock(
       const PString & token,     ///<  Token to identify connection
-      PSafetyMode mode = PSafeReadWrite,
-      SIP_PDU::StatusCodes * errorCode = NULL
+      PSafetyMode mode = PSafeReadWrite,  ///< Lock mode
+      SIP_PDU::StatusCodes * errorCode = NULL ///< Error code if lock fails
     );
 
     virtual PBoolean IsAcceptedAddress(const SIPURL & toAddr);
@@ -390,7 +392,8 @@ class SIPEndPoint : public OpalRTPEndPoint
      */
     bool Register(
       const SIPRegister::Params & params, ///< Registration parameters
-      PString & aor                       ///< Resultant address-of-record for unregister
+      PString & aor,                      ///< Resultant address-of-record for unregister
+      bool asynchronous = true            ///< Wait for completion, may take some time
     );
 
     /// Registration function for backward compatibility.
@@ -573,15 +576,18 @@ class SIPEndPoint : public OpalRTPEndPoint
        avoided.
       */
     bool Unsubscribe(
-      const PString & aor          ///< Unique token for registration
+      const PString & aor,             ///< Unique token for registration
+      bool invalidateNotifiers = false ///< Notifiers in SIPSubscribe::Params are to be reset
     );
     bool Unsubscribe(
       SIPSubscribe::PredefinedPackages eventPackage,  ///< Event package being unsubscribed
-      const PString & aor                             ///< Address-of-record for subscription
+      const PString & aor,                            ///< Address-of-record for subscription
+      bool invalidateNotifiers = false                ///< Notifiers in SIPSubscribe::Params are to be reset
     );
     bool Unsubscribe(
       const PString & eventPackage,  ///< Event package being unsubscribed
-      const PString & aor            ///< Address-of-record for subscription
+      const PString & aor,             ///< Address-of-record for subscription
+      bool invalidateNotifiers = false ///< Notifiers in SIPSubscribe::Params are to be reset
     );
 
     /**Unsubscribe all current subscriptions.
@@ -628,7 +634,7 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
 
     virtual void OnSubscriptionStatus(
-      SIPSubscribeHandler & handler, /// event subscription paramaters
+      SIPSubscribeHandler & handler, ///< Event subscription paramaters
       const SIPURL & uri,            ///< Target URI for the subscription.
       bool wasSubscribing,           ///< Indication the subscribing or unsubscribing
       bool reSubscribing,            ///< If subscribing then indication was refeshing subscription
@@ -660,26 +666,42 @@ class SIPEndPoint : public OpalRTPEndPoint
       const SIPDialogNotification & info  ///< Information on dialog state change
     );
 
-    /**Send SIP message
+
+    /**Send IM text
      */
-    virtual bool Message(
-      const PURL & to, 
-      const PString & type,
-      const PString & body,
-      PURL & from,
-      PString & conversationId
+    virtual PBoolean Message(
+      OpalIM & message
     );
 
-    /**Callback called when a message sent by the endpoint didn't reach
-     * its destination or when the proxy or remote endpoint returns
-     * an error code.
+    /**Send SIP message
      */
-    virtual void OnMessageFailed(
-      const SIPURL & messageUrl,
+    bool SendMESSAGE(
+      SIPMessage::Params & params
+    );
+
+    /**Callback called when a message completes in any manner
+     */
+    virtual void OnMESSAGECompleted(
+      const SIPMessage::Params & params,
       SIP_PDU::StatusCodes reason
     );
 
 
+    /**Send SIP OPTIONS
+     */
+    virtual bool SendOPTIONS(
+      const SIPOptions::Params & params
+    );
+
+    /**Callback called when an OPTIONS command is completed, either
+       successfully or with error.
+     */
+    virtual void OnOptionsCompleted(
+      const SIPOptions::Params & params,    ///< Original parameter for SendOPTIONS() call
+      const SIP_PDU & response              ///< Response packet, check GetStatusCode() for result
+    );
+
+
     /**Publish new state information.
        A expire time of zero will cease to automatically update the publish.
      */
@@ -728,6 +750,13 @@ class SIPEndPoint : public OpalRTPEndPoint
       const PURL & to
     );
 
+    /**Get default mode for PRACK support.
+      */
+    SIPConnection::PRACKMode GetDefaultPRACKMode() const { return m_defaultPrackMode; }
+
+    /**Set default mode for PRACK support.
+      */
+    void SetDefaultPRACKMode(SIPConnection::PRACKMode mode) { m_defaultPrackMode = mode; }
 
     void SetMIMEForm(PBoolean v) { mimeForm = v; }
     PBoolean GetMIMEForm() const { return mimeForm; }
@@ -814,19 +843,18 @@ class SIPEndPoint : public OpalRTPEndPoint
     virtual SIPURL GetDefaultRegisteredPartyName(const OpalTransport & transport);
     
 
-    /**Return the contact URL for the given host and user name
-     * based on the listening port of the registration to that host.
-     * 
-     * That URL can be used as as contact field in outgoing
-     * requests.
-     *
-     * The URL is translated if required.
-     *
-     * If no active registration is used, return the result of GetLocalURL
-     * on the given transport.
+    /**Adjust the outgoing PDU to registered information.
+       Various header fields of the PDU must be adjusted to agree with values
+       provided to/from the active registration for the domain the call is being
+       made to. For example the "From" field must agree exactly with
+     
+       If no active registration is available, the result of GetLocalURL() on
+       the given transport is set to the Contact field.
      */
-    SIPURL GetContactURL(const OpalTransport &transport, const SIPURL & localURI);
-
+    void AdjustToRegistration(
+      const OpalTransport & transport,
+      SIP_PDU & pdu
+    );
 
     /**Return the local URL for the given transport and user name.
      * That URL can be used as via address, and as contact field in outgoing
@@ -906,10 +934,6 @@ class SIPEndPoint : public OpalRTPEndPoint
 
     virtual void OnStartTransaction(SIPConnection & conn, SIPTransaction & transaction);
 
-#if OPAL_HAS_SIPIM
-    virtual OpalSIPIMManager & GetSIPIMManager() { return m_sipIMManager; }
-#endif
-
     void UpdateHandlerIndexes(SIPHandler * handler) { activeSIPHandlers.Update(handler); }
 
   protected:
@@ -919,6 +943,8 @@ class SIPEndPoint : public OpalRTPEndPoint
     SIPURL        proxy;
     PString       userAgentString;
 
+    SIPConnection::PRACKMode m_defaultPrackMode;
+
     bool          mimeForm;
     unsigned      maxRetries;
     PTimeInterval retryTimeoutMin;   // T1
@@ -943,47 +969,34 @@ class SIPEndPoint : public OpalRTPEndPoint
     PAtomicInteger          lastSentCSeq;
     int                     m_defaultAppearanceCode;
 
-  public:
-    class WorkThreadPool;
+    std::map<PString, PSyncPoint> m_registrationComplete;
+
+
+    // Thread pooling
     class SIP_Work
     {
       public:
         SIP_Work(SIPEndPoint & ep, SIP_PDU * pdu, const PString & token);
         virtual ~SIP_Work();
 
-        virtual void Process();
+        virtual void Work();
 
         SIPEndPoint & m_endpoint;
         SIP_PDU     * m_pdu;
         PString       m_token;
     };
 
-    class WorkThreadPool : public PThreadPool<SIP_Work>
+    class WorkThreadPool : public PQueuedThreadPool<SIP_Work>
     {
       public:
+        WorkThreadPool(unsigned maxWorkers)
+          : PQueuedThreadPool<SIP_Work>(maxWorkers)
+        { }
         virtual WorkerThreadBase * CreateWorkerThread();
     } m_connectionThreadPool, m_handlerThreadPool;
 
-  protected:
-    typedef std::queue<SIP_Work *> SIP_WorkQueue;
-
-    class SIP_Work_Thread : public WorkThreadPool::WorkerThread
-    {
-      public:
-        SIP_Work_Thread(WorkThreadPool & pool_);
-
-        void AddWork(SIP_Work * work);
-        void RemoveWork(SIP_Work * work);
-        unsigned GetWorkSize() const;
-
-        void Main();
-        void Shutdown();
-
-      protected:
-        PSyncPoint m_sync;
-        SIP_WorkQueue m_pduQueue;
-    };
 
+    // Network interface checking
     enum {
       HighPriority = 80,
       LowPriority  = 30,
@@ -1006,15 +1019,12 @@ class SIPEndPoint : public OpalRTPEndPoint
     friend void InterfaceMonitor::OnAddInterface(const PIPSocket::InterfaceEntry & entry);
     friend void InterfaceMonitor::OnRemoveInterface(const PIPSocket::InterfaceEntry & entry);
 
-#if OPAL_HAS_SIPIM
-    OpalSIPIMManager m_sipIMManager;
-#endif
-
     bool m_disableTrying;
 
     P_REMOVE_VIRTUAL_VOID(OnReceivedIntervalTooBrief(SIPTransaction &, SIP_PDU &));
     P_REMOVE_VIRTUAL_VOID(OnReceivedAuthenticationRequired(SIPTransaction &, SIP_PDU &));
     P_REMOVE_VIRTUAL_VOID(OnReceivedOK(SIPTransaction &, SIP_PDU &));
+    P_REMOVE_VIRTUAL_VOID(OnMessageFailed(const SIPURL &, SIP_PDU::StatusCodes));
 
   public:
     struct ConnectionlessMessageInfo {
