@@ -25,15 +25,13 @@
 
  */
 
+#define _CRT_SECURE_NO_WARNINGS
+
+#include <codec/opalplugin.hpp>
+
 #include "h264pipe_win32.h"
 
-#ifdef _MSC_VER
-#include "../common/rtpframe.h"
-#include "../common/trace.h"
-#else
 #include "rtpframe.h"
-#include "trace.h"
-#endif
 
 #include <string.h>
 #include <stdio.h>
@@ -43,10 +41,14 @@
 
 
 #include <fstream>
+#include <iostream>
+#include <iomanip>
+using namespace std;
+
 
 #define PIPE_TIMEOUT 5000
 #define BUFSIZE 4096
-#define GPL_PROCESS_FILENAME "plugins\\h264_video_pwplugin_helper.exe"
+#define GPL_PROCESS_FILENAME "x264plugin_helper.exe"
 #define DIR_SEPERATOR "\\"
 #define DIR_TOKENISER ";"
 
@@ -68,10 +70,12 @@ H264EncCtx::~H264EncCtx()
   closeAndRemovePipes();
 }
 
-bool H264EncCtx::Load()
+bool H264EncCtx::Load(void * instance)
 {
+  if (loaded)
+    return true;
 
-  snprintf(pipeName, sizeof(pipeName), "\\\\.\\pipe\\x264-%d", GetCurrentProcessId());
+  snprintf(pipeName, sizeof(pipeName), "\\\\.\\pipe\\x264-%d-%p", GetCurrentProcessId(), instance);
 
   if (!createPipes()) {
   
@@ -81,7 +85,7 @@ bool H264EncCtx::Load()
   
   if (!findGplProcess()) { 
 
-    TRACE(1, "H264\tIPC\tPP: Couldn't find GPL process executable " << GPL_PROCESS_FILENAME);
+    PTRACE(1, "x264", "IPC\tPP: Couldn't find GPL process executable " << GPL_PROCESS_FILENAME);
     closeAndRemovePipes(); 
     return false;
   }  
@@ -95,13 +99,14 @@ bool H264EncCtx::Load()
   if (!ConnectNamedPipe(stream, NULL)) {
 	if (GetLastError() != ERROR_PIPE_CONNECTED) {
 
-          TRACE(1, "H264\tIPC\tPP: Could not establish communication with child process (" << ErrorMessage() << ")");
+          PTRACE(1, "x264", "IPC\tPP: Could not establish communication with child process (" << ErrorMessage() << ")");
 	  closeAndRemovePipes(); 
 	  return false;
 	}
   } 
 
-  TRACE(1, "H264\tIPC\tPP: Successfully established communication with child process");
+  PTRACE(1, "x264", "IPC\tPP: Successfully established communication with child process");
+  loaded = true;
   return true;
 }
 
@@ -184,7 +189,7 @@ bool H264EncCtx::createPipes()
 
   if (stream == INVALID_HANDLE_VALUE) {
 
-    TRACE(1, "H264\tIPC\tPP: Failure on creating Pipe - terminating (" << ErrorMessage() << ")");
+    PTRACE(1, "x264", "IPC\tPP: Failure on creating Pipe - terminating (" << ErrorMessage() << ")");
     return false;
   }
   return true;
@@ -193,9 +198,9 @@ bool H264EncCtx::createPipes()
 void H264EncCtx::closeAndRemovePipes()
 {
   if (!DisconnectNamedPipe(stream))
- 	TRACE(1, "H264\tIPC\tPP: Failure on disconnecting Pipe (" << ErrorMessage() << ")");
+ 	PTRACE(1, "x264", "IPC\tPP: Failure on disconnecting Pipe (" << ErrorMessage() << ")");
   if (!CloseHandle(stream))
- 	TRACE(1, "H264\tIPC\tPP: Failure on closing Handle (" << ErrorMessage() << ")");
+ 	PTRACE(1, "x264", "IPC\tPP: Failure on closing Handle (" << ErrorMessage() << ")");
 }
 
 void H264EncCtx::readStream (LPVOID data, unsigned bytes)
@@ -212,13 +217,13 @@ void H264EncCtx::readStream (LPVOID data, unsigned bytes)
 
   if (!fSuccess) {
 
-	TRACE(1, "H264\tIPC\tPP: Failure on reading - terminating (" << ErrorMessage() << ")");
+	PTRACE(1, "x264", "IPC\tPP: Failure on reading - terminating (" << ErrorMessage() << ")");
 	closeAndRemovePipes();
   }
 
   if (bytes != bytesRead) {
 
-    TRACE(1, "H264\tIPC\tPP: Failure on reading - terminating (Read " << bytesRead << " bytes, expected " << bytes);
+    PTRACE(1, "x264", "IPC\tPP: Failure on reading - terminating (Read " << bytesRead << " bytes, expected " << bytes);
 	closeAndRemovePipes();
   }
 }
@@ -237,13 +242,13 @@ void H264EncCtx::writeStream (LPCVOID data, unsigned bytes)
 
   if (!fSuccess) {
 
-	TRACE(1, "H264\tIPC\tPP: Failure on writing - terminating (" << ErrorMessage() << ")");
+	PTRACE(1, "x264", "IPC\tPP: Failure on writing - terminating (" << ErrorMessage() << ")");
 	closeAndRemovePipes();
   }
 
   if (bytes != bytesWritten) {
 
-    TRACE(1, "H264\tIPC\tPP: Failure on writing - terminating (Written " << bytesWritten << " bytes, intended " << bytes);
+    PTRACE(1, "x264", "IPC\tPP: Failure on writing - terminating (Written " << bytesWritten << " bytes, intended " << bytes);
 	closeAndRemovePipes();
   }
 }
@@ -252,27 +257,32 @@ void H264EncCtx::flushStream ()
 {
   if (!FlushFileBuffers(stream)) {
 
-	TRACE(1, "H264\tIPC\tPP: Failure on flushing - terminating (" << ErrorMessage() << ")");
+	PTRACE(1, "x264", "IPC\tPP: Failure on flushing - terminating (" << ErrorMessage() << ")");
 	closeAndRemovePipes();
   }
 }
 
 bool H264EncCtx::findGplProcess()
 {
-  char * env = ::getenv("PWLIBPLUGINDIR");
+  static char const DefaultPath[] = "." DIR_TOKENISER "C:\\PTLib_Plugins";
+  char buffer[sizeof(DefaultPath)];
+
+  char * env = ::getenv("PTLIBPLUGINDIR");
   if (env == NULL)
-    env = ::getenv("PTLIBPLUGINDIR");
-  if (env != NULL) {
-    const char * token = strtok(env, DIR_TOKENISER);
-    while (token != NULL) {
-
-      if (checkGplProcessExists(token)) 
-        return true;
-
-      token = strtok(NULL, DIR_TOKENISER);
-    }
+    env = ::getenv("PWLIBPLUGINDIR");
+  if (env == NULL) {
+    strcpy(buffer, DefaultPath);
+    env = buffer;
   }
-  return checkGplProcessExists(".");
+
+  const char * token = strtok(env, DIR_TOKENISER);
+  while (token != NULL) {
+    if (checkGplProcessExists(token))
+      return true;
+    token = strtok(NULL, DIR_TOKENISER);
+  }
+
+  return false;
 }
 
 bool H264EncCtx::checkGplProcessExists (const char * dir)
@@ -286,12 +296,12 @@ bool H264EncCtx::checkGplProcessExists (const char * dir)
 
   fin.open(gplProcess,ios::in);
   if( !fin.is_open() ){
-    TRACE(1, "H264\tIPC\tPP: Couldn't find GPL process executable in " << gplProcess);
+    PTRACE(1, "x264", "IPC\tPP: Couldn't find GPL process executable in " << gplProcess);
     fin.close();
     return false;
   }
   fin.close();
-  TRACE(1, "H264\tIPC\tPP: Found GPL process executable in " << gplProcess);
+  PTRACE(1, "x264", "IPC\tPP: Found GPL process executable in " << gplProcess);
   return true;
 }
 
@@ -312,16 +322,16 @@ bool H264EncCtx::execGplProcess()
                       NULL,        // Process handle not inheritable
                       NULL,        // Thread handle not inheritable
                       FALSE,       // Set handle inheritance to FALSE
-                      (Trace::GetLevel() == 0) ? CREATE_NO_WINDOW : 0, // Creation flags
+                      CREATE_NO_WINDOW, // Creation flags
                       NULL,        // Use parent's environment block
                       NULL,        // Use parent's starting directory 
                       &si,         // Pointer to STARTUPINFO structure
                       &pi ))       // Pointer to PROCESS_INFORMATION structure
   {
-      TRACE(1, "H264\tIPC\tPP: Couldn't create child process: " << ErrorMessage());
+      PTRACE(1, "x264", "IPC\tPP: Couldn't create child process: " << ErrorMessage());
       return false;
   }
-  TRACE(1, "H264\tIPC\tPP: Successfully created child process " << pi.dwProcessId);
+  PTRACE(1, "x264", "IPC\tPP: Successfully created child process " << pi.dwProcessId);
   return true;
 }
 
