@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24101 $
+ * $Revision: 26320 $
  * $Author: rjongbloed $
- * $Date: 2010-03-04 04:36:04 -0600 (Thu, 04 Mar 2010) $
+ * $Date: 2011-08-17 06:58:09 -0500 (Wed, 17 Aug 2011) $
  */
 
 #include <ptlib.h>
@@ -96,7 +96,6 @@ OpalVideoTranscoder::OpalVideoTranscoder(const OpalMediaFormat & inputMediaForma
   : OpalTranscoder(inputMediaFormat, outputMediaFormat)
   , inDataSize(10*1024)
   , outDataSize(10*1024)
-  , videoEncoderMaxOutputSize(10*1024)
   , forceIFrame(false)
   , lastFrameWasIFrame(false)
   , m_totalFrames(0)
@@ -112,7 +111,7 @@ static void SetFrameBytes(const OpalMediaFormat & fmt, const PString & widthOpti
   int height = fmt.GetOptionInteger(heightOption, PVideoFrameInfo::CIFHeight);
   int newSize = PVideoDevice::CalculateFrameBytes(width, height, fmt);
   if (newSize > 0)
-    size = newSize;
+    size = sizeof(PluginCodec_Video_FrameHeader) + newSize;
 }
 
 
@@ -126,7 +125,7 @@ bool OpalVideoTranscoder::UpdateMediaFormats(const OpalMediaFormat & input, cons
   SetFrameBytes(inputMediaFormat,  OpalVideoFormat::MaxRxFrameWidthOption(), OpalVideoFormat::MaxRxFrameHeightOption(), inDataSize);
   SetFrameBytes(outputMediaFormat, OpalVideoFormat::FrameWidthOption(),      OpalVideoFormat::FrameHeightOption(),      outDataSize);
 
-  videoEncoderMaxOutputSize = outputMediaFormat.GetOptionInteger(OpalVideoFormat::MaxTxPacketSizeOption(), outDataSize);
+  outputMediaFormat.SetOptionInteger(OpalMediaFormat::MaxTxPacketSizeOption(), maxOutputSize);
 
   return PTrue;
 }
@@ -179,25 +178,25 @@ PString OpalVideoUpdatePicture::GetName() const
   return "Update Picture";
 }
 
-PString OpalVideoUpdatePicture2::GetName() const
+
+OpalVideoPictureLoss::OpalVideoPictureLoss(unsigned sequenceNumber, unsigned timestamp)
+  : m_sequenceNumber(sequenceNumber)
+  , m_timestamp(timestamp)
 {
-  return "Update Picture 2";
 }
 
+
+PString OpalVideoPictureLoss::GetName() const
+{
+  return "Picture Loss";
+}
+
+
 PString OpalTemporalSpatialTradeOff::GetName() const
 {
   return "Temporal Spatial Trade Off";
 }
 
-PString OpalLostPartialPicture::GetName() const
-{
-  return "Lost Partial Picture";
-}
-
-PString OpalLostPicture::GetName() const
-{
-  return "Lost Picture";
-}
 
 #endif // OPAL_VIDEO
 
