@@ -25,9 +25,9 @@
  *
  * The author of this code is Derek J Smithies
  *
- * $Revision: 23926 $
+ * $Revision: 25336 $
  * $Author: rjongbloed $
- * $Date: 2010-01-12 18:44:33 -0600 (Tue, 12 Jan 2010) $
+ * $Date: 2011-03-16 03:51:39 -0500 (Wed, 16 Mar 2011) $
  */
 
 #include <ptlib.h>
@@ -80,11 +80,11 @@ IAX2Connection::IAX2Connection(OpalCall & call,               /* Owner call for
   PStringArray res = IAX2EndPoint::DissectRemoteParty(inRemoteParty);
   remotePartyNumber = res[IAX2EndPoint::extensionIndex];
 
-   SetCallToken(token);
+  SetCallToken(token);
   originating = PFalse;
 
   ep.CopyLocalMediaFormats(localMediaFormats);
-  AdjustMediaFormats(true, localMediaFormats, NULL);
+  AdjustMediaFormats(true, NULL, localMediaFormats);
   PTRACE(5, "Local ordered codecs are " << localMediaFormats);
   
   local_hold = PFalse;
@@ -165,7 +165,7 @@ void IAX2Connection::AnsweringCall(AnswerCallResponse response)
   PTRACE(3, "IAX2Con\tAnswering call: " << response);
 
   PSafeLockReadWrite safeLock(*this);
-  if (!safeLock.IsLocked() || GetPhase() >= ReleasingPhase)
+  if (!safeLock.IsLocked() || IsReleased())
     return;
 
   OpalConnection::AnsweringCall(response);
@@ -174,25 +174,24 @@ void IAX2Connection::AnsweringCall(AnswerCallResponse response)
 
 PBoolean IAX2Connection::SetConnected()
 {
-    PTRACE(3, "IAX2Con\t SET CONNECTED " 
+  PTRACE(3, "IAX2Con\t SET CONNECTED " 
 	 << PString(IsOriginating() ? " Originating" : "Receiving"));
   ////if no media streams, try and start them
-    
+  
+  if (!IsOriginating())
+    iax2Processor.SendAnswerMessageToRemoteNode();
+  
   if (mediaStreams.IsEmpty()) {
-    if (!IsOriginating())
-      iax2Processor.SendAnswerMessageToRemoteNode();
-
+    
     ownerCall.OpenSourceMediaStreams(*this, OpalMediaType::Audio(), 1);
     PSafePtr<OpalConnection> otherParty = GetOtherPartyConnection();
     if (otherParty != NULL)
       ownerCall.OpenSourceMediaStreams(*otherParty, OpalMediaType::Audio(), 1);
-
-    
+        
     jitterBuffer.SetDelay(endpoint.GetManager().GetMinAudioJitterDelay() * 8, 
 			  endpoint.GetManager().GetMaxAudioJitterDelay() * 8);
     PTRACE(5, "Iax2Con\t Start jitter buffer");
-  }
-
+  }  
   return OpalConnection::SetConnected();
 }
 
@@ -208,7 +207,6 @@ PBoolean IAX2Connection::SetAlerting(const PString & PTRACE_PARAM(calleeName), P
     if (GetPhase() == AlertingPhase)
       return PFalse;
     
-    alertingTime = PTime();
     SetPhase(AlertingPhase);
   }
 
@@ -258,18 +256,6 @@ PBoolean IAX2Connection::SendUserInputString(const PString & value)
   return OpalConnection::SendUserInputString(value);
 }
 
-OpalConnection::SendUserInputModes IAX2Connection::GetRealSendUserInputMode() const
-{
-  switch (sendUserInputMode) {
-    case SendUserInputAsString:
-    case SendUserInputAsTone:
-      return sendUserInputMode;
-    default:
-      break;
-  }
-
-  return SendUserInputAsTone;
-}
   
 PBoolean IAX2Connection::SendUserInputTone(char tone, unsigned /*duration*/ ) 
 { 
@@ -362,7 +348,7 @@ void IAX2Connection::BuildRemoteCapabilityTable(unsigned int remoteCapability, u
     remoteMediaFormats.Reorder(PStringArray(wildcard));
   }
 
-  AdjustMediaFormats(false, remoteMediaFormats, NULL);
+  AdjustMediaFormats(false, NULL, remoteMediaFormats);
   PTRACE(4, "Connection\tREMOTE Codecs are " << remoteMediaFormats);
 }
 
@@ -412,44 +398,38 @@ unsigned int IAX2Connection::ChooseCodec()
   return 0;
 }
 
-PBoolean IAX2Connection::IsConnectionOnHold(bool fromRemote)
+PBoolean IAX2Connection::IsOnHold(bool fromRemote)
 {
   return fromRemote ? remote_hold : local_hold;
 }
 
-bool IAX2Connection::HoldConnection()
+bool IAX2Connection::Hold(bool fromRemote, bool placeOnHold)
 {
-  if (local_hold)
+  if (IsOnHold(fromRemote))
     return true;
-    
-  local_hold = PTrue;
-  PauseMediaStreams(PTrue);
-  endpoint.OnHold(*this);
-  
-  iax2Processor.SendHold();
+
+  if (fromRemote)
+    return false;
+
+  local_hold = placeOnHold;
+  PauseMediaStreams(placeOnHold);
+  OnHold(fromRemote, placeOnHold);
+
+  if (placeOnHold)
+    iax2Processor.SendHold();
+  else
+    iax2Processor.SendHoldRelease();
   return true;
 }
 
-bool IAX2Connection::RetrieveConnection()
-{
-  if (!local_hold)
-    return true;
-  
-  local_hold = PFalse;
-  PauseMediaStreams(PFalse);  
-  endpoint.OnHold(*this);
-  
-  iax2Processor.SendHoldRelease();
-  return true;
-}
 
 void IAX2Connection::RemoteHoldConnection()
 {
   if (remote_hold)
     return;
     
-  remote_hold = PTrue;
-  endpoint.OnHold(*this);
+  remote_hold = true;
+  OnHold(true, true);
 }
 
 void IAX2Connection::RemoteRetrieveConnection()
@@ -457,8 +437,8 @@ void IAX2Connection::RemoteRetrieveConnection()
   if (!remote_hold)
     return;
     
-  remote_hold = PFalse;    
-  endpoint.OnHold(*this);
+  remote_hold = false;    
+  endpoint.OnHold(*this, true, false);
 }
 
 bool IAX2Connection::TransferConnection(const PString & remoteParty)
@@ -516,8 +496,8 @@ void IAX2Connection::ReceivedSoundPacketFromNetwork(IAX2Frame *soundFrame)
 
 PBoolean IAX2Connection::ReadSoundPacket(RTP_DataFrame & packet)
 { 
-  if (GetPhase() >= ReleasingPhase)/*We are hanging up. */
-    return PFalse;            /* Sending sound now is meaningless */
+  if (IsReleased()) /*We are hanging up. */
+    return false;   /* Sending sound now is meaningless */
   
   PTRACE(6, "Iax2Con\t Start read from  jitter buffer"); 
   if (!jitterBuffer.ReadData(packet)) {
@@ -534,11 +514,9 @@ PBoolean IAX2Connection::ReadSoundPacket(RTP_DataFrame & packet)
 
 #endif // OPAL_IAX2
 
-/* The comment below is magic for those who use emacs to edit this file. */
-/* With the comment below, the tab key does auto indent to 2 spaces.     */
-
-
-/*
+/* The comment below is magic for those who use emacs to edit this file.
+ * With the comment below, the tab key does auto indent to 2 spaces.    
+ *
  * Local Variables:
  * mode:c
  * c-basic-offset:2
