@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24547 $
+ * $Revision: 25336 $
  * $Author: rjongbloed $
- * $Date: 2010-06-30 01:36:31 -0500 (Wed, 30 Jun 2010) $
+ * $Date: 2011-03-16 03:51:39 -0500 (Wed, 16 Mar 2011) $
  */
 
 #include <ptlib.h>
@@ -83,7 +83,7 @@ OpalCall::~OpalCall()
   delete m_recordManager;
 #endif
 
-  PTRACE(3, "Call\t" << *this << " destroyed.");
+  PTRACE(3, "Call\tDestroyed " << *this);
 }
 
 
@@ -95,6 +95,7 @@ void OpalCall::PrintOn(ostream & strm) const
 
 void OpalCall::OnEstablishedCall()
 {
+  PTRACE(3, "Call\tEstablished " << *this);
   manager.OnEstablishedCall(*this);
 }
 
@@ -285,7 +286,7 @@ PBoolean OpalCall::OnEstablished(OpalConnection & connection)
 
 PSafePtr<OpalConnection> OpalCall::GetOtherPartyConnection(const OpalConnection & connection) const
 {
-  PTRACE(3, "Call\tGetOtherPartyConnection " << connection);
+  PTRACE(4, "Call\tGetOtherPartyConnection " << connection);
 
   PSafePtr<OpalConnection> otherConnection;
   EnumerateConnections(otherConnection, PSafeReference, &connection);
@@ -301,7 +302,7 @@ bool OpalCall::Hold()
 
   PSafePtr<OpalConnection> connection;
   while (EnumerateConnections(connection, PSafeReadWrite)) {
-    if (connection->HoldConnection())
+    if (connection->IsNetworkConnection() && connection->Hold(false, true))
       ok = true;
   }
 
@@ -317,7 +318,7 @@ bool OpalCall::Retrieve()
 
   PSafePtr<OpalConnection> connection;
   while (EnumerateConnections(connection, PSafeReadWrite)) {
-    if (connection->RetrieveConnection())
+    if (connection->IsNetworkConnection() && connection->Hold(false, false))
       ok = true;
   }
 
@@ -329,7 +330,7 @@ bool OpalCall::IsOnHold() const
 {
   PSafePtr<OpalConnection> connection;
   while (EnumerateConnections(connection, PSafeReadOnly)) {
-    if (connection->IsConnectionOnHold(false) || connection->IsConnectionOnHold(true))
+    if (connection->IsNetworkConnection() && connection->IsOnHold(false))
       return true;
   }
 
@@ -339,7 +340,10 @@ bool OpalCall::IsOnHold() const
 
 bool OpalCall::Transfer(const PString & newAddress, OpalConnection * connection)
 {
-  PCaselessString prefix = newAddress.Left(newAddress.Find(':'));
+  PCaselessString prefix;
+  PINDEX colon = newAddress.Find(':');
+  if (colon != P_MAX_INDEX)
+    prefix = newAddress.Left(colon);
 
   if (connection == NULL) {
     for (PSafePtr<OpalConnection> conn = GetConnection(0); conn != NULL; ++conn) {
@@ -354,7 +358,7 @@ bool OpalCall::Transfer(const PString & newAddress, OpalConnection * connection)
   if (prefix == "*")
     return connection->TransferConnection(connection->GetPrefixName() + newAddress.Mid(1));
 
-  if (prefix == connection->GetPrefixName() || manager.HasCall(newAddress))
+  if (prefix.IsEmpty() || prefix == connection->GetPrefixName() || manager.HasCall(newAddress))
     return connection->TransferConnection(newAddress);
 
   PTRACE(3, "Call\tTransferring " << *connection << " to \"" << newAddress << '"');
@@ -384,20 +388,18 @@ bool OpalCall::Transfer(const PString & newAddress, OpalConnection * connection)
 }
 
 
-OpalMediaFormatList OpalCall::GetMediaFormats(const OpalConnection & connection,
-                                              PBoolean includeSpecifiedConnection)
+OpalMediaFormatList OpalCall::GetMediaFormats(const OpalConnection & connection)
 {
   OpalMediaFormatList commonFormats;
 
-  PBoolean first = PTrue;
+  bool first = true;
 
   PSafePtr<OpalConnection> otherConnection;
-  while (EnumerateConnections(otherConnection, PSafeReadOnly, includeSpecifiedConnection ? NULL : &connection)) {
+  while (EnumerateConnections(otherConnection, PSafeReadOnly, &connection)) {
     OpalMediaFormatList possibleFormats = OpalTranscoder::GetPossibleFormats(otherConnection->GetMediaFormats());
-    connection.AdjustMediaFormats(true, possibleFormats, otherConnection);
     if (first) {
       commonFormats = possibleFormats;
-      first = PFalse;
+      first = false;
     }
     else {
       // Want intersection of the possible formats for all connections.
@@ -410,7 +412,13 @@ OpalMediaFormatList OpalCall::GetMediaFormats(const OpalConnection & connection,
     }
   }
 
-  connection.AdjustMediaFormats(true, commonFormats, NULL);
+  // If nothing comes out of the above, e.g. when this is called very early
+  // in the call sequence and there is no second connection in call yet, then
+  // return all the possible media formats.
+  if (first)
+    commonFormats = OpalTranscoder::GetPossibleFormats(manager.GetCommonMediaFormats(false, true));
+
+  connection.AdjustMediaFormats(true, NULL, commonFormats);
 
   PTRACE(4, "Call\tGetMediaFormats for " << connection << '\n'
          << setfill('\n') << commonFormats << setfill(' '));
@@ -419,6 +427,14 @@ OpalMediaFormatList OpalCall::GetMediaFormats(const OpalConnection & connection,
 }
 
 
+void OpalCall::AdjustMediaFormats(bool local, const OpalConnection & connection, OpalMediaFormatList & mediaFormats) const
+{
+  PSafePtr<OpalConnection> otherConnection;
+  while (EnumerateConnections(otherConnection, PSafeReadOnly, &connection))
+    otherConnection->AdjustMediaFormats(local, &connection, mediaFormats);
+}
+
+
 PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection, 
                                      const OpalMediaType & mediaType,
                                                   unsigned sessionID, 
@@ -456,7 +472,13 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
       sinkStream = patch->GetSink();
     if (sourceStream->GetMediaFormat() == preselectedFormat ||
         (sinkStream != NULL && sinkStream->GetMediaFormat() == preselectedFormat)) {
-      PTRACE(3, "Call\tOpenSourceMediaStreams (already opened)" << traceText);
+      if (sourceStream->IsPaused()) {
+        sourceStream->SetPaused(false);
+        PTRACE(3, "Call\tOpenSourceMediaStreams (un-pausing)" << traceText);
+      }
+      else {
+        PTRACE(3, "Call\tOpenSourceMediaStreams (already opened)" << traceText);
+      }
       return true;
     }
     if (sinkStream == NULL && sourceStream->IsOpen()) {
@@ -473,7 +495,6 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
 
   // Create the sinks and patch if needed
   bool startedOne = false;
-  OpalMediaPatch * patch = NULL;
   OpalMediaFormat sourceFormat, sinkFormat;
 
   // Reorder destinations so we give preference to symmetric codecs
@@ -488,7 +509,6 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
     order += '@' + mediaType;        // And media of the same type
 
     OpalMediaFormatList sinkMediaFormats = otherConnection->GetMediaFormats();
-    connection.AdjustMediaFormats(false, sinkMediaFormats, otherConnection);
     if (sinkMediaFormats.IsEmpty()) {
       PTRACE(2, "Call\tOpenSourceMediaStreams failed with no sink formats" << traceText);
       return false;
@@ -504,7 +524,6 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
       sourceMediaFormats = sourceFormat; // Use the source format already established
     else {
       sourceMediaFormats = connection.GetMediaFormats();
-      otherConnection->AdjustMediaFormats(false, sourceMediaFormats, &connection);
       if (sourceMediaFormats.IsEmpty()) {
         PTRACE(2, "Call\tOpenSourceMediaStreams failed with no source formats" << traceText);
         return false;
@@ -548,7 +567,6 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
 #endif
 
     OpalMediaFormatList localMediaFormats = OpalMediaFormat::GetAllRegisteredMediaFormats();
-    otherConnection->AdjustMediaFormats(true, localMediaFormats, &connection);
     if (!SelectMediaFormats(mediaType,
                             sourceMediaFormats,
                             sinkMediaFormats,
@@ -577,13 +595,19 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
     sinkStream = otherConnection->OpenMediaStream(sinkFormat, sessionID, false);
     if (sinkStream != NULL) {
       startedOne = true;
+      if (!sourceStream.SetSafetyMode(PSafeReadOnly))
+        return false;
+
+      OpalMediaPatch * patch = sourceStream->GetPatch();
       if (patch == NULL) {
         patch = manager.CreateMediaPatch(*sourceStream, sinkStream->RequiresPatchThread(sourceStream) &&
                                                         sourceStream->RequiresPatchThread(sinkStream));
         if (patch == NULL)
           return false;
       }
+
       patch->AddSink(sinkStream);
+      sourceStream.SetSafetyMode(PSafeReference);
     }
   }
 
@@ -593,10 +617,18 @@ PBoolean OpalCall::OpenSourceMediaStreams(OpalConnection & connection,
     return false;
   }
 
-  if (patch != NULL) {
-    // if a patch was created, make sure the callback is called, just once per connection
-    while (EnumerateConnections(otherConnection, PSafeReadWrite))
-      otherConnection->OnPatchMediaStream(otherConnection == &connection, *patch);
+  // if a patch was created, make sure the callback is called, just once per
+  // connection. Note must lock connection before stream or we can deadlock.
+  while (EnumerateConnections(otherConnection, PSafeReadWrite)) {
+    if (!sourceStream.SetSafetyMode(PSafeReadOnly))
+      return false;
+
+    OpalMediaPatch * patch = sourceStream->GetPatch();
+    if (patch == NULL)
+      return false;
+
+    otherConnection->OnPatchMediaStream(otherConnection == &connection, *patch);
+    sourceStream.SetSafetyMode(PSafeReference);
   }
 
   return true;
