@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24705 $
+ * $Revision: 26099 $
  * $Author: rjongbloed $
- * $Date: 2010-09-15 19:08:56 -0500 (Wed, 15 Sep 2010) $
+ * $Date: 2011-06-29 22:59:39 -0500 (Wed, 29 Jun 2011) $
  */
 
 #include <ptlib.h>
@@ -47,6 +47,7 @@
 #include <codec/silencedetect.h>
 #include <codec/echocancel.h>
 #include <codec/rfc2833.h>
+#include <codec/g711codec.h>
 #include <codec/vidcodec.h>
 #include <rtp/rtp.h>
 #include <t120/t120proto.h>
@@ -149,7 +150,7 @@ ostream & operator<<(ostream & o, OpalConnection::SendUserInputModes m)
     "SendUserInputAsString",
     "SendUserInputAsTone",
     "SendUserInputAsRFC2833",
-    "SendUserInputAsSeparateRFC2833",
+    "SendUserInputInBand",
     "SendUserInputAsProtocolDefault"
   };
 
@@ -212,12 +213,9 @@ OpalConnection::OpalConnection(OpalCall & call,
   : PSafeObject(&call)  // Share the lock flag from the call
   , ownerCall(call)
   , endpoint(ep)
-  , phase(UninitialisedPhase)
+  , m_phase(UninitialisedPhase)
   , callToken(token)
   , originating(PFalse)
-  , alertingTime(0)
-  , connectedTime(0)
-  , callEndTime(0)
   , productInfo(ep.GetProductInfo())
   , localPartyName(ep.GetDefaultLocalPartyName())
   , displayName(ep.GetDefaultDisplayName())
@@ -234,17 +232,11 @@ OpalConnection::OpalConnection(OpalCall & call,
 #ifdef _MSC_VER
 #pragma warning(disable:4355)
 #endif
-  , m_dtmfNotifier(PCREATE_NOTIFIER(OnDetectInBandDTMF))
-#ifdef _MSC_VER
-#pragma warning(default:4355)
-#endif
+  , m_dtmfDetectNotifier(PCREATE_NOTIFIER(OnDetectInBandDTMF))
   , m_sendInBandDTMF(true)
-  , m_installedInBandDTMF(false)
   , m_emittedInBandDTMF(0)
 #endif
-#ifdef _MSC_VER
-#pragma warning(disable:4355)
-#endif
+  , m_dtmfSendNotifier(PCREATE_NOTIFIER(OnSendInBandDTMF))
 #if OPAL_HAS_MIXER
   , m_recordAudioNotifier(PCREATE_NOTIFIER(OnRecordAudio))
 #if OPAL_VIDEO
@@ -260,6 +252,9 @@ OpalConnection::OpalConnection(OpalCall & call,
 #if OPAL_FAX
   , m_faxMediaStreamsSwitchState(e_NotSwitchingFaxMediaStreams)
 #endif
+#if P_LUA
+  , m_luaScriptLoaded(false)
+#endif
 {
   PTRACE(3, "OpalCon\tCreated connection " << *this);
 
@@ -268,7 +263,7 @@ OpalConnection::OpalConnection(OpalCall & call,
   ownerCall.connectionsActive.Append(this);
 
   if (stringOptions != NULL)
-    m_connStringOptions = *stringOptions;
+    m_stringOptions = *stringOptions;
 
   minAudioJitterDelay = endpoint.GetManager().GetMinAudioJitterDelay();
   maxAudioJitterDelay = endpoint.GetManager().GetMaxAudioJitterDelay();
@@ -298,7 +293,7 @@ OpalConnection::OpalConnection(OpalCall & call,
       sendUserInputMode = SendUserInputAsTone;
       break;
     case SendDTMFAsRFC2833:
-      sendUserInputMode = SendUserInputAsInlineRFC2833;
+      sendUserInputMode = SendUserInputAsRFC2833;
       break;
     case SendDTMFAsDefault:
     default:
@@ -306,10 +301,22 @@ OpalConnection::OpalConnection(OpalCall & call,
       break;
   }
 
-#if OPAL_HAS_IM
+#if 0 // OPAL_HAS_IM
   m_rfc4103Context[0].SetMediaFormat(OpalT140);
   m_rfc4103Context[1].SetMediaFormat(OpalT140);
 #endif
+
+#if P_LUA
+  BindToInstance(m_lua, "connection");
+  m_lua.SetFunction("PTRACE", PLua::TraceFunction);
+  m_lua.SetValue("token",  callToken);
+
+  PString str = GetPrefixName();
+  m_lua.SetValue("prefix", str);
+  m_lua.CallLuaFunction("OnConstruct");
+#endif
+
+  m_phaseTime[UninitialisedPhase].SetCurrentTime();
 }
 
 OpalConnection::~OpalConnection()
@@ -324,6 +331,11 @@ OpalConnection::~OpalConnection()
   delete t120handler;
 #endif
 
+#if P_LUA
+  m_lua.CallLuaFunction("OnDestroy");
+  UnbindFromInstance(m_lua, "call");
+#endif
+
   ownerCall.connectionsActive.Remove(this);
   ownerCall.SafeDereference();
 
@@ -350,19 +362,13 @@ PBoolean OpalConnection::OnSetUpConnection()
 }
 
 
-bool OpalConnection::HoldConnection()
+bool OpalConnection::Hold(bool /*fromRemote*/, bool /*placeOnHold*/)
 {
   return false;
 }
 
 
-bool OpalConnection::RetrieveConnection()
-{
-  return false;
-}
-
-
-PBoolean OpalConnection::IsConnectionOnHold(bool /*fromRemote*/)
+PBoolean OpalConnection::IsOnHold(bool /*fromRemote*/)
 {
   return false;
 }
@@ -424,17 +430,16 @@ bool OpalConnection::TransferConnection(const PString & PTRACE_PARAM(remoteParty
 void OpalConnection::Release(CallEndReason reason)
 {
   {
-    PWaitAndSignal m(phaseMutex);
-    if (phase >= ReleasingPhase) {
-      PTRACE(2, "OpalCon\tAlready released " << *this);
+    PWaitAndSignal mutex(m_phaseMutex);
+    if (IsReleased()) {
+      PTRACE(3, "OpalCon\tAlready released " << *this);
       return;
     }
     SetPhase(ReleasingPhase);
   }
 
-  {
-    PSafeLockReadWrite safeLock(*this);
-    if (!safeLock.IsLocked()) {
+  if (synchronousOnRelease) {
+    if (!LockReadWrite()) {
       PTRACE(2, "OpalCon\tAlready released " << *this);
       return;
     }
@@ -444,24 +449,31 @@ void OpalConnection::Release(CallEndReason reason)
     // Now set reason for the connection close
     SetCallEndReason(reason);
 
-    if (synchronousOnRelease) {
-      OnReleased();
-      return;
-    }
+    UnlockReadWrite();
 
-    // Add a reference for the thread we are about to start
-    SafeReference();
+    OnReleased();
+    return;
   }
 
-  PThread::Create(PCREATE_NOTIFIER(OnReleaseThreadMain), 0,
+  PTRACE(3, "OpalCon\tReleasing asynchronously " << *this);
+
+  // Add a reference for the thread we are about to start
+  SafeReference();
+  PThread::Create(PCREATE_NOTIFIER(OnReleaseThreadMain), reason.AsInteger(),
                   PThread::AutoDeleteThread,
                   PThread::NormalPriority,
                   "OnRelease");
 }
 
 
-void OpalConnection::OnReleaseThreadMain(PThread &, INT)
+void OpalConnection::OnReleaseThreadMain(PThread &, INT reason)
 {
+  if (LockReadWrite()) {
+    // Now set reason for the connection close
+    SetCallEndReason(CallEndReason(reason));
+    UnlockReadWrite();
+  }
+
   OnReleased();
 
   PTRACE(4, "OpalCon\tOnRelease thread completed for " << *this);
@@ -473,11 +485,32 @@ void OpalConnection::OnReleaseThreadMain(PThread &, INT)
 
 void OpalConnection::OnReleased()
 {
-  PTRACE(3, "OpalCon\tOnReleased " << *this);
+  PTRACE(4, "OpalCon\tOnReleased " << *this);
+
+  CloseMediaStreams();
 
   endpoint.OnReleased(*this);
 
-  CloseMediaStreams();
+  SetPhase(ReleasedPhase);
+
+#if PTRACING
+  static const int Level = 3;
+  if (PTrace::CanTrace(Level)) {
+    ostream & trace = PTrace::Begin(Level, __FILE__, __LINE__);
+    trace << "OpalCon\tConnection " << *this << " released\n"
+             "        Initial Time: " << m_phaseTime[UninitialisedPhase] << '\n';
+    for (Phases ph = SetUpPhase; ph < NumPhases; ph = (Phases)(ph+1)) {
+      trace << setw(20) << ph << ": ";
+      if (m_phaseTime[ph].IsValid())
+        trace << (m_phaseTime[ph]-m_phaseTime[UninitialisedPhase]);
+      else
+        trace << "N/A";
+      trace << '\n';
+    }
+    trace << "     Call end reason: " << callEndReason << '\n'
+          << PTrace::End;
+  }
+#endif
 }
 
 
@@ -559,10 +592,8 @@ PBoolean OpalConnection::SetConnected()
 {
   PTRACE(3, "OpalCon\tSetConnected for " << *this);
 
-  if (GetPhase() < ConnectedPhase) {
+  if (GetPhase() < ConnectedPhase)
     SetPhase(ConnectedPhase);
-    connectedTime = PTime();
-  }
 
   if (!mediaStreams.IsEmpty() && GetPhase() < EstablishedPhase) {
     SetPhase(EstablishedPhase);
@@ -576,7 +607,6 @@ PBoolean OpalConnection::SetConnected()
 void OpalConnection::OnConnectedInternal()
 {
   if (GetPhase() < ConnectedPhase) {
-    connectedTime = PTime();
     SetPhase(ConnectedPhase);
     OnConnected();
   }
@@ -603,14 +633,48 @@ void OpalConnection::OnEstablished()
 }
 
 
-void OpalConnection::AdjustMediaFormats(bool local, OpalMediaFormatList & mediaFormats, OpalConnection * otherConnection) const
+bool OpalConnection::OnTransferNotify(const PStringToString & info)
 {
-  if (otherConnection != NULL && otherConnection != this)
-    otherConnection->AdjustMediaFormats(local, mediaFormats, NULL);
-  else {
-    mediaFormats.Remove(m_connStringOptions(OPAL_OPT_REMOVE_CODEC).Lines());
-    endpoint.AdjustMediaFormats(local, *this, mediaFormats);
+  return endpoint.OnTransferNotify(*this, info);
+}
+
+
+void OpalConnection::AdjustMediaFormats(bool   local,
+                        const OpalConnection * otherConnection,
+                         OpalMediaFormatList & mediaFormats) const
+{
+  if (otherConnection != NULL)
+    return;
+
+  mediaFormats.Remove(m_stringOptions(OPAL_OPT_REMOVE_CODEC).Lines());
+
+  if (local) {
+    for (PINDEX i = 0; i < m_stringOptions.GetSize(); ++i) {
+      PString key = m_stringOptions.GetKeyAt(i);
+      PINDEX colon = key.Find(':');
+      if (colon != P_MAX_INDEX) {
+        PString fmtName = key.Left(colon);
+        PString optName = key.Mid(colon+1);
+        if (!fmtName.IsEmpty() && !optName.IsEmpty()) {
+          PString optValue = m_stringOptions.GetDataAt(i);
+          OpalMediaFormatList::const_iterator iterFormat;
+          while ((iterFormat = mediaFormats.FindFormat(fmtName, iterFormat)) != mediaFormats.end()) {
+            OpalMediaFormat & format = const_cast<OpalMediaFormat &>(*iterFormat);
+            if (format.SetOptionValue(optName, optValue)) {
+              PTRACE(4, "OpalCon\tSet media format " << format
+                     << " option " << optName << " to \"" << optValue << '"');
+            }
+            else {
+              PTRACE(2, "OpalCon\tFailed to set media format " << format
+                     << " option " << optName << " to \"" << optValue << '"');
+            }
+          }
+        }
+      }
+    }
   }
+
+  endpoint.AdjustMediaFormats(local, *this, mediaFormats);
 }
 
 
@@ -722,21 +786,7 @@ bool OpalConnection::CloseMediaStream(unsigned sessionId, bool source)
 
 bool OpalConnection::CloseMediaStream(OpalMediaStream & stream)
 {
-  if (!stream.Close())
-    return false;
-
-  if (stream.IsSource())
-    return true;
-
-  PSafePtr<OpalConnection> otherConnection = GetOtherPartyConnection();
-  if (otherConnection == NULL)
-    return true;
-
-  OpalMediaStreamPtr otherStream = otherConnection->GetMediaStream(stream.GetSessionID(), true);
-  if (otherStream == NULL)
-    return true;
-
-  return otherStream->Close();
+  return stream.Close();
 }
 
 
@@ -776,48 +826,51 @@ void OpalConnection::CloseMediaStreams()
 }
 
 
-void OpalConnection::PauseMediaStreams(PBoolean paused)
+void OpalConnection::PauseMediaStreams(bool paused)
 {
   for (OpalMediaStreamPtr mediaStream = mediaStreams; mediaStream != NULL; ++mediaStream)
     mediaStream->SetPaused(paused);
 }
 
 
-OpalMediaStream * OpalConnection::CreateMediaStream(
-#if OPAL_VIDEO
-  const OpalMediaFormat & mediaFormat,
-  unsigned sessionID,
-  PBoolean isSource
-#else
-  const OpalMediaFormat & ,
-  unsigned ,
-  PBoolean 
-#endif
-  )
+void OpalConnection::OnPauseMediaStream(OpalMediaStream & /*strm*/, bool /*paused*/)
 {
+}
+
+
 #if OPAL_VIDEO
+OpalMediaStream * OpalConnection::CreateMediaStream(const OpalMediaFormat & mediaFormat,
+                                                    unsigned sessionID,
+                                                    PBoolean isSource)
+{
   if (mediaFormat.GetMediaType() == OpalMediaType::Video()) {
     if (isSource) {
       PVideoInputDevice * videoDevice;
-      PBoolean autoDelete;
-      if (CreateVideoInputDevice(mediaFormat, videoDevice, autoDelete)) {
+      PBoolean autoDeleteGrabber;
+      if (CreateVideoInputDevice(mediaFormat, videoDevice, autoDeleteGrabber)) {
         PVideoOutputDevice * previewDevice;
-        if (!CreateVideoOutputDevice(mediaFormat, PTrue, previewDevice, autoDelete))
+        PBoolean autoDeletePreview;
+        if (!CreateVideoOutputDevice(mediaFormat, PTrue, previewDevice, autoDeletePreview))
           previewDevice = NULL;
-        return new OpalVideoMediaStream(*this, mediaFormat, sessionID, videoDevice, previewDevice, autoDelete);
+        return new OpalVideoMediaStream(*this, mediaFormat, sessionID, videoDevice, previewDevice, autoDeleteGrabber, autoDeletePreview);
       }
     }
     else {
       PVideoOutputDevice * videoDevice;
       PBoolean autoDelete;
       if (CreateVideoOutputDevice(mediaFormat, PFalse, videoDevice, autoDelete))
-        return new OpalVideoMediaStream(*this, mediaFormat, sessionID, NULL, videoDevice, autoDelete);
+        return new OpalVideoMediaStream(*this, mediaFormat, sessionID, NULL, videoDevice, false, autoDelete);
     }
   }
-#endif
 
   return NULL;
 }
+#else
+OpalMediaStream * OpalConnection::CreateMediaStream(const OpalMediaFormat &, unsigned, PBoolean)
+{
+  return NULL;
+}
+#endif
 
 
 PBoolean OpalConnection::OnOpenMediaStream(OpalMediaStream & stream)
@@ -841,9 +894,33 @@ PBoolean OpalConnection::OnOpenMediaStream(OpalMediaStream & stream)
 
 void OpalConnection::OnClosedMediaStream(const OpalMediaStream & stream)
 {
+  OpalMediaPatch * patch = stream.GetPatch();
+  if (patch != NULL) {
 #if OPAL_HAS_MIXER
-  OnStopRecording(stream.GetPatch());
+    OnStopRecording(patch);
 #endif
+
+    if (silenceDetector != NULL && patch->RemoveFilter(silenceDetector->GetReceiveHandler(), m_filterMediaFormat)) {
+      PTRACE(4, "OpalCon\tRemoved silence detect filter on connection " << *this << ", patch " << patch);
+    }
+
+#if OPAL_AEC
+    if (echoCanceler != NULL && patch->RemoveFilter(stream.IsSource() ? echoCanceler->GetReceiveHandler()
+                                                  : echoCanceler->GetSendHandler(), m_filterMediaFormat)) {
+      PTRACE(4, "OpalCon\tRemoved echo canceler filter on connection " << *this << ", patch " << patch);
+    }
+#endif
+
+#if OPAL_PTLIB_DTMF
+    if (patch->RemoveFilter(m_dtmfDetectNotifier, OpalPCM16)) {
+      PTRACE(4, "OpalCon\tRemoved detect DTMF filter on connection " << *this << ", patch " << patch);
+    }
+    if (!m_dtmfSendFormat.IsEmpty() && patch->RemoveFilter(m_dtmfSendNotifier, m_dtmfSendFormat)) {
+      PTRACE(4, "OpalCon\tRemoved DTMF send filter on connection " << *this << ", patch " << patch);
+    }
+#endif
+  }
+
   endpoint.OnClosedMediaStream(stream);
 }
 
@@ -865,7 +942,7 @@ void OpalConnection::OnStartRecording(OpalMediaPatch * patch)
     return;
   }
 
-  patch->AddFilter(m_recordAudioNotifier, OPAL_PCM16);
+  patch->AddFilter(m_recordAudioNotifier, OpalPCM16);
 #if OPAL_VIDEO
   patch->AddFilter(m_recordVideoNotifier, OPAL_YUV420P);
 #endif
@@ -881,7 +958,7 @@ void OpalConnection::OnStopRecording(OpalMediaPatch * patch)
 
   ownerCall.OnStopRecording(MakeRecordingKey(*patch));
 
-  patch->RemoveFilter(m_recordAudioNotifier, OPAL_PCM16);
+  patch->RemoveFilter(m_recordAudioNotifier, OpalPCM16);
 #if OPAL_VIDEO
   patch->RemoveFilter(m_recordVideoNotifier, OPAL_YUV420P);
 #endif
@@ -895,31 +972,41 @@ void OpalConnection::OnPatchMediaStream(PBoolean isSource, OpalMediaPatch & patc
 {
   patch.SetCommandNotifier(PCREATE_NOTIFIER(OnMediaCommand), !isSource);
 
-  OpalMediaFormat mediaFormat = patch.GetSource().GetMediaFormat();
+  OpalMediaFormat mediaFormat = isSource ? patch.GetSource().GetMediaFormat() : patch.GetSink()->GetMediaFormat();
+
   if (mediaFormat.GetMediaType() == OpalMediaType::Audio()) {
-    PTRACE(3, "OpalCon\tAdding audio filters to patch " << patch);
+    if (!mediaFormat.IsTransportable()) {
+      m_filterMediaFormat = mediaFormat;
+
+      if (isSource && silenceDetector != NULL) {
+        silenceDetector->SetParameters(endpoint.GetManager().GetSilenceDetectParams(), mediaFormat.GetClockRate());
+        patch.AddFilter(silenceDetector->GetReceiveHandler(), mediaFormat);
+        PTRACE(4, "OpalCon\tAdded silence detect filter on connection " << *this << ", patch " << patch);
+      }
 
-    if (isSource && silenceDetector != NULL) {
-      silenceDetector->SetParameters(endpoint.GetManager().GetSilenceDetectParams(), mediaFormat.GetClockRate());
-      patch.AddFilter(silenceDetector->GetReceiveHandler(), mediaFormat);
-    }
 #if OPAL_AEC
-    if (echoCanceler) {
-      echoCanceler->SetParameters(endpoint.GetManager().GetEchoCancelParams());
-      echoCanceler->SetClockRate(mediaFormat.GetClockRate());
-      patch.AddFilter(isSource ? echoCanceler->GetReceiveHandler() : echoCanceler->GetSendHandler(), OpalPCM16);
-    }
+      if (echoCanceler) {
+        echoCanceler->SetParameters(endpoint.GetManager().GetEchoCancelParams());
+        echoCanceler->SetClockRate(mediaFormat.GetClockRate());
+        patch.AddFilter(isSource ? echoCanceler->GetReceiveHandler()
+                                 : echoCanceler->GetSendHandler(), mediaFormat);
+        PTRACE(4, "OpalCon\tAdded echo canceler filter on connection " << *this << ", patch " << patch);
+      }
 #endif
+    }
 
 #if OPAL_PTLIB_DTMF
     if (m_detectInBandDTMF && isSource) {
-      patch.AddFilter(m_dtmfNotifier, OPAL_PCM16);
+      patch.AddFilter(m_dtmfDetectNotifier, OpalPCM16);
       PTRACE(4, "OpalCon\tAdded detect DTMF filter on connection " << *this << ", patch " << patch);
     }
 
     if (m_sendInBandDTMF && !isSource) {
-      m_installedInBandDTMF = true;
-      patch.AddFilter(PCREATE_NOTIFIER(OnSendInBandDTMF), OPAL_PCM16);
+      if (mediaFormat == OpalG711_ULAW_64K || mediaFormat == OpalG711_ALAW_64K)
+        m_dtmfSendFormat = mediaFormat;
+      else
+        m_dtmfSendFormat = OpalPCM16;
+      patch.AddFilter(m_dtmfSendNotifier, mediaFormat);
       PTRACE(4, "OpalCon\tAdded send DTMF filter on connection " << *this << ", patch " << patch);
     }
 #endif
@@ -1053,7 +1140,7 @@ PBoolean OpalConnection::CreateVideoOutputDevice(const OpalMediaFormat & mediaFo
 }
 
 
-bool OpalConnection::SendVideoUpdatePicture(unsigned sessionID, int firstGOB, int firstMB, int numBlocks) const
+bool OpalConnection::SendVideoUpdatePicture(unsigned sessionID, bool force) const
 {
   PSafeLockReadWrite safeLock(*this);
   if (!safeLock.IsLocked())
@@ -1065,10 +1152,12 @@ bool OpalConnection::SendVideoUpdatePicture(unsigned sessionID, int firstGOB, in
     PTRACE(3, "OpalCon\tNo video stream do video update picture in connection " << *this);
     return false;
   }
-  
-  OpalVideoUpdatePicture updatePictureCommand(firstGOB, firstMB, numBlocks);
-  stream->ExecuteCommand(updatePictureCommand);
-  PTRACE(3, "OpalCon\tUpdate video picture (I-frame) requested in video stream " << *stream);
+
+  if (force)
+    stream->ExecuteCommand(OpalVideoUpdatePicture());
+  else
+    stream->ExecuteCommand(OpalVideoPictureLoss());
+  PTRACE(3, "OpalCon\tUpdate video picture (I-Frame) requested in video stream " << *stream);
 
   return true;
 }
@@ -1177,15 +1266,41 @@ PBoolean OpalConnection::SendUserInputString(const PString & value)
 #if OPAL_PTLIB_DTMF
 PBoolean OpalConnection::SendUserInputTone(char tone, unsigned duration)
 {
-  if (!m_installedInBandDTMF)
+  if (m_dtmfSendFormat.IsEmpty())
     return false;
 
   if (duration <= 0)
     duration = PDTMFEncoder::DefaultToneLen;
 
   PTRACE(3, "OPAL\tSending in-band DTMF tone '" << tone << "', duration=" << duration);
+
+  PDTMFEncoder dtmfSamples;
+  dtmfSamples.AddTone(tone, duration);
+  PINDEX size = dtmfSamples.GetSize();
+
   m_inBandMutex.Wait();
-  m_inBandDTMF.AddTone(tone, duration);
+
+  switch (m_dtmfSendFormat.GetPayloadType()) {
+    case RTP_DataFrame::PCMU :
+      if (m_inBandDTMF.SetSize(size)) {
+        for (PINDEX i = 0; i < size; ++i)
+          m_inBandDTMF[i] = (BYTE)Opal_PCM_G711_uLaw::ConvertSample(dtmfSamples[i]);
+      }
+      break;
+
+    case RTP_DataFrame::PCMA :
+      if (m_inBandDTMF.SetSize(size)) {
+        for (PINDEX i = 0; i < size; ++i)
+          m_inBandDTMF[i] = (BYTE)Opal_PCM_G711_ALaw::ConvertSample(dtmfSamples[i]);
+      }
+      break;
+
+    default :
+      size *= sizeof(short);
+      if (m_inBandDTMF.SetSize(size))
+        memcpy(m_inBandDTMF.GetPointer(), dtmfSamples, size);
+  }
+
   m_inBandMutex.Signal();
 
   return true;
@@ -1213,9 +1328,7 @@ void OpalConnection::OnUserInputTone(char tone, unsigned duration)
 PString OpalConnection::GetUserInput(unsigned timeout)
 {
   PString reply;
-  if (userInputAvailable.Wait(PTimeInterval(0, timeout)) &&
-      GetPhase() < ReleasingPhase &&
-      LockReadWrite()) {
+  if (userInputAvailable.Wait(PTimeInterval(0, timeout)) && !IsReleased() && LockReadWrite()) {
     reply = userInputString;
     userInputString = PString();
     UnlockReadWrite();
@@ -1275,15 +1388,15 @@ void OpalConnection::OnSendInBandDTMF(RTP_DataFrame & frame, INT)
   // Can't do the usual LockReadWrite() as deadlocks
   m_inBandMutex.Wait();
 
-  PINDEX bytes = (m_inBandDTMF.GetSize() - m_emittedInBandDTMF)*sizeof(short);
+  PINDEX bytes = m_inBandDTMF.GetSize() - m_emittedInBandDTMF;
   if (bytes > frame.GetPayloadSize())
     bytes = frame.GetPayloadSize();
   memcpy(frame.GetPayloadPtr(), &m_inBandDTMF[m_emittedInBandDTMF], bytes);
 
-  m_emittedInBandDTMF += bytes/sizeof(short);
+  m_emittedInBandDTMF += bytes;
 
   if (m_emittedInBandDTMF >= m_inBandDTMF.GetSize()) {
-    PTRACE(4, "OPAL\tSent in-band DTMF tone, " << m_inBandDTMF.GetSize() << " samples");
+    PTRACE(4, "OPAL\tSent in-band DTMF tone, " << m_inBandDTMF.GetSize() << " bytes");
     m_inBandDTMF.SetSize(0);
     m_emittedInBandDTMF = 0;
   }
@@ -1313,7 +1426,7 @@ PString OpalConnection::GetLocalPartyURL() const
 
 bool OpalConnection::IsPresentationBlocked() const
 {
-  return m_connStringOptions.GetBoolean(OPAL_OPT_PRESENTATION_BLOCK);
+  return m_stringOptions.GetBoolean(OPAL_OPT_PRESENTATION_BLOCK);
 }
 
 
@@ -1371,6 +1484,12 @@ bool OpalConnection::SetAlertingType(const PString & /*info*/)
 }
 
 
+PString OpalConnection::GetCallInfo() const
+{
+  return PString::Empty();
+}
+
+
 void OpalConnection::SetAudioJitterDelay(unsigned minDelay, unsigned maxDelay)
 {
   maxDelay = PMAX(10, PMIN(maxDelay, 999));
@@ -1398,69 +1517,96 @@ PINDEX OpalConnection::GetMaxRtpPayloadSize() const
 
 void OpalConnection::SetPhase(Phases phaseToSet)
 {
-  PTRACE(3, "OpalCon\tSetPhase from " << phase << " to " << phaseToSet << " for " << *this);
+  PTRACE(3, "OpalCon\tSetPhase from " << m_phase << " to " << phaseToSet << " for " << *this);
 
-  PWaitAndSignal m(phaseMutex);
+  PWaitAndSignal mutex(m_phaseMutex);
 
   // With next few lines we will prevent phase to ever go down when it
   // reaches ReleasingPhase - end result - once you call Release you never
   // go back.
-  if (phase < ReleasingPhase || (phase == ReleasingPhase && phaseToSet == ReleasedPhase))
-    phase = phaseToSet;
+  if (m_phase < ReleasingPhase || (m_phase == ReleasingPhase && phaseToSet == ReleasedPhase)) {
+    m_phase = phaseToSet;
+    if (!m_phaseTime[m_phase].IsValid())
+      m_phaseTime[m_phase].SetCurrentTime();
+  }
 }
 
 
 void OpalConnection::SetStringOptions(const StringOptions & options, bool overwrite)
 {
   if (overwrite)
-    m_connStringOptions = options;
+    m_stringOptions = options;
   else {
     for (PINDEX i = 0; i < options.GetSize(); ++i)
-      m_connStringOptions.SetAt(options.GetKeyAt(i), options.GetDataAt(i));
+      m_stringOptions.SetAt(options.GetKeyAt(i), options.GetDataAt(i));
   }
 }
 
 void OpalConnection::OnApplyStringOptions()
 {
-  endpoint.GetManager().OnApplyStringOptions(*this, m_connStringOptions);
-}
+  endpoint.GetManager().OnApplyStringOptions(*this, m_stringOptions);
 
-void OpalConnection::ApplyStringOptions(OpalConnection::StringOptions & stringOptions)
-{
-  PTRACE(4, "OpalCon\tApplying string options:\n" << stringOptions);
+  PTRACE_IF(4, !m_stringOptions.IsEmpty(), "OpalCon\tApplying string options:\n" << m_stringOptions);
 
   if (LockReadWrite()) {
     PCaselessString str;
 
-    m_connStringOptions = stringOptions;
+    str = m_stringOptions(OPAL_OPT_USER_INPUT_MODE);
+    if (str == "RFC2833")
+      SetSendUserInputMode(SendUserInputAsRFC2833);
+    else if (str == "String")
+      SetSendUserInputMode(SendUserInputAsString);
+    else if (str == "Tone")
+      SetSendUserInputMode(SendUserInputAsTone);
+    else if (str == "Q.931")
+      SetSendUserInputMode(SendUserInputAsQ931);
+    else if (str == "InBand") {
+      SetSendUserInputMode(SendUserInputInBand);
+      m_sendInBandDTMF = true;
+    }
 
 #if OPAL_PTLIB_DTMF
-    m_sendInBandDTMF   = stringOptions.GetBoolean(OPAL_OPT_ENABLE_INBAND_DTMF, m_sendInBandDTMF);
-    m_detectInBandDTMF = stringOptions.GetBoolean(OPAL_OPT_DETECT_INBAND_DTMF, m_detectInBandDTMF);
-    m_sendInBandDTMF   = stringOptions.GetBoolean(OPAL_OPT_SEND_INBAND_DTMF,   m_sendInBandDTMF);
+    m_sendInBandDTMF   = m_stringOptions.GetBoolean(OPAL_OPT_ENABLE_INBAND_DTMF, m_sendInBandDTMF);
+    m_detectInBandDTMF = m_stringOptions.GetBoolean(OPAL_OPT_DETECT_INBAND_DTMF, m_detectInBandDTMF);
+    m_sendInBandDTMF   = m_stringOptions.GetBoolean(OPAL_OPT_SEND_INBAND_DTMF,   m_sendInBandDTMF);
 
-    m_dtmfScaleMultiplier = stringOptions.GetInteger(OPAL_OPT_DTMF_MULT, m_dtmfScaleMultiplier);
-    m_dtmfScaleDivisor    = stringOptions.GetInteger(OPAL_OPT_DTMF_DIV,  m_dtmfScaleDivisor);
+    m_dtmfScaleMultiplier = m_stringOptions.GetInteger(OPAL_OPT_DTMF_MULT, m_dtmfScaleMultiplier);
+    m_dtmfScaleDivisor    = m_stringOptions.GetInteger(OPAL_OPT_DTMF_DIV,  m_dtmfScaleDivisor);
 #endif
 
-    m_autoStartInfo.Initialise(stringOptions);
+    m_autoStartInfo.Initialise(m_stringOptions);
 
-    if (stringOptions.GetBoolean(OPAL_OPT_DISABLE_JITTER))
+    if (m_stringOptions.GetBoolean(OPAL_OPT_DISABLE_JITTER))
       maxAudioJitterDelay = minAudioJitterDelay = 0;
     else {
-      maxAudioJitterDelay = stringOptions.GetInteger(OPAL_OPT_MAX_JITTER, maxAudioJitterDelay);
-      minAudioJitterDelay = stringOptions.GetInteger(OPAL_OPT_MIN_JITTER, minAudioJitterDelay);
+      maxAudioJitterDelay = m_stringOptions.GetInteger(OPAL_OPT_MAX_JITTER, maxAudioJitterDelay);
+      minAudioJitterDelay = m_stringOptions.GetInteger(OPAL_OPT_MIN_JITTER, minAudioJitterDelay);
     }
 
 #if OPAL_HAS_MIXER
-    if (stringOptions.Contains(OPAL_OPT_RECORD_AUDIO))
-      m_recordingFilename = m_connStringOptions(OPAL_OPT_RECORD_AUDIO);
+    if (m_stringOptions.Contains(OPAL_OPT_RECORD_AUDIO))
+      m_recordingFilename = m_stringOptions(OPAL_OPT_RECORD_AUDIO);
 #endif
 
-    str = stringOptions(OPAL_OPT_ALERTING_TYPE);
+    str = m_stringOptions(OPAL_OPT_ALERTING_TYPE);
     if (!str.IsEmpty())
       SetAlertingType(str);
 
+#if P_LUA
+    if (!m_luaScriptLoaded) {
+      str = m_stringOptions("script");
+      bool ok = true;
+      if (!str.IsEmpty()) 
+        ok = m_lua.LoadString(str);
+      else {
+        str = m_stringOptions("scriptfile");
+        if (!str.IsEmpty()) 
+          m_lua.LoadFile(str);
+      }
+    }
+    m_lua.CallLuaFunction("OnSetOptions");
+#endif
+
     UnlockReadWrite();
   }
 }
@@ -1474,18 +1620,26 @@ OpalMediaFormatList OpalConnection::GetMediaFormats() const
 
 OpalMediaFormatList OpalConnection::GetLocalMediaFormats()
 {
-  return ownerCall.GetMediaFormats(*this, FALSE);
+  if (m_localMediaFormats.IsEmpty())
+    m_localMediaFormats = ownerCall.GetMediaFormats(*this);
+  return m_localMediaFormats;
 }
 
 
 void OpalConnection::OnStartMediaPatch(OpalMediaPatch & patch)
 {
+#if P_LUA
+  m_lua.CallLuaFunction("OnStartMediaPatch");
+#endif
   GetEndPoint().GetManager().OnStartMediaPatch(*this, patch);
 }
 
 
 void OpalConnection::OnStopMediaPatch(OpalMediaPatch & patch)
 {
+#if P_LUA
+  m_lua.CallLuaFunction("OnStopMediaPatch");
+#endif
   GetEndPoint().GetManager().OnStopMediaPatch(*this, patch);
 }
 
@@ -1541,7 +1695,7 @@ void OpalConnection::AutoStartMap::Initialise(const OpalConnection::StringOption
             SetAutoStart(mediaType, OpalMediaType::Receive);
           else if (tokens[i] *= "sendonly")
             SetAutoStart(mediaType, OpalMediaType::Transmit);
-          else if (tokens[i] *= "offer")
+          else if ((tokens[i] *= "offer") || (tokens[i] *= "inactive"))
             SetAutoStart(mediaType, OpalMediaType::OfferInactive);
           else if (tokens[i] *= "exclusive") {
             OpalMediaTypeFactory::KeyList_T types = OpalMediaType::GetList();
@@ -1594,7 +1748,7 @@ OpalMediaType::AutoStartMode OpalConnection::AutoStartMap::GetAutoStart(const Op
 }
 
 
-#if OPAL_HAS_IM
+#if 0 //OPAL_HAS_IM
 
 bool OpalConnection::TransmitInternalIM(const OpalMediaFormat & format, RTP_IMFrame & frame)
 {
@@ -1622,9 +1776,35 @@ void OpalConnection::OnReceiveInternalIM(const OpalMediaFormat & format, RTP_IMF
 }
 
 
-bool OpalConnection::TransmitExternalIM(const OpalMediaFormat & format, RTP_IMFrame & frame)
+bool OpalConnection::TransmitExternalIM(const OpalMediaFormat & /*format*/, RTP_IMFrame & frame)
 {
-  return endpoint.TransmitExternalIM(*this, format, frame);
+  PURL remotePartyURL, localPartyURL;
+  PString remotePartyName;
+
+  // get information about sender
+  PSafePtr<OpalConnection> otherParty = GetOtherPartyConnectionAs<OpalConnection>();
+  if (otherParty != NULL) {
+    remotePartyURL  = otherParty->GetRemotePartyCallbackURL();
+    remotePartyName = otherParty->GetRemotePartyName();
+    localPartyURL   = otherParty->GetLocalPartyURL();
+  }
+
+  // pass information up the chain
+
+  OpalIM message;
+  message.m_conversationId = GetToken();
+  message.m_to             = localPartyURL;
+  message.m_from           = remotePartyURL;
+  message.m_fromName       = remotePartyName;
+  message.m_mimeType       = frame.GetContentType();
+
+  T140String t140;
+  frame.GetContent(t140);
+  t140.AsString(message.m_body);
+
+  endpoint.OnMessageReceived(message);
+
+  return true;
 }
 
 bool OpalConnection::OnReceiveExternalIM(const OpalMediaFormat & format, RTP_IMFrame & body)
@@ -1635,38 +1815,26 @@ bool OpalConnection::OnReceiveExternalIM(const OpalMediaFormat & format, RTP_IMF
 #endif
 
 
-bool OpalConnection::StringOptions::GetBoolean(const char * key, bool dflt) const
-{
-  PString * value = GetAt(key);
-  if (value == NULL)
-    return dflt;
-
-  return value->IsEmpty() || (*value *= "true") || (*value *= "yes") || value->AsInteger() != 0;
-}
-
-
-long OpalConnection::StringOptions::GetInteger(const char * key, long dflt) const
-{
-  PString * value = GetAt(key);
-  if (value == NULL)
-    return dflt;
-
-  return value->AsInteger();
-}
-
-
 void OpalConnection::StringOptions::ExtractFromURL(PURL & url)
 {
   PStringToString params = url.GetParamVars();
   params.MakeUnique();
   for (PINDEX i = 0; i < params.GetSize(); ++i) {
     PCaselessString key = params.GetKeyAt(i);
-    if (key.NumCompare("OPAL-") == EqualTo) {
+    if (key.NumCompare(OPAL_URL_PARAM_PREFIX) == EqualTo) {
       SetAt(key.Mid(5), params.GetDataAt(i));
       url.SetParamVar(key, PString::Empty());
     }
   }
 }
 
+#if P_LUA
+
+int OpalConnection::LuaSetOption(lua_State * /*lua*/)
+{
+  return 0;
+}
+
+#endif
 
 /////////////////////////////////////////////////////////////////////////////
