@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24787 $
+ * $Revision: 26069 $
  * $Author: rjongbloed $
- * $Date: 2010-10-11 17:52:01 -0500 (Mon, 11 Oct 2010) $
+ * $Date: 2011-06-23 19:56:50 -0500 (Thu, 23 Jun 2011) $
  */
 
 #include <ptlib.h>
@@ -241,8 +241,10 @@ PBoolean OpalLocalConnection::SetUpConnection()
 
     OnApplyStringOptions();
 
-    if (!endpoint.OnIncomingCall(*this))
+    if (!endpoint.OnIncomingCall(*this)) {
+      Release(EndedByLocalBusy);
       return false;
+    }
 
     if (!endpoint.IsDeferredAlerting())
       AlertingIncoming();
