@@ -25,9 +25,9 @@
  * Contributor(s): Craig Southeren (craigs@postincrement.com)
  *                 Robert Jongbloed (robertj@voxlucida.com.au)
  *
- * $Revision: 23957 $
+ * $Revision: 25300 $
  * $Author: rjongbloed $
- * $Date: 2010-01-21 01:36:07 -0600 (Thu, 21 Jan 2010) $
+ * $Date: 2011-03-09 13:22:53 -0600 (Wed, 09 Mar 2011) $
  */
 
 #include <ptlib.h>
@@ -535,6 +535,10 @@ const short * OpalAudioMixer::AudioStream::GetAudioDataPtr()
 OpalVideoMixer::OpalVideoMixer(Styles style, unsigned width, unsigned height, unsigned rate, bool pushThread)
   : OpalBaseMixer(pushThread, 1000/rate, OpalMediaFormat::VideoClockRate/rate)
   , m_style(style)
+  , m_bgFillRed(0)
+  , m_bgFillGreen(0)
+  , m_bgFillBlue(0)
+  , m_lastStreamCount(0)
 {
   SetFrameSize(width, height);
 }
@@ -562,7 +566,7 @@ bool OpalVideoMixer::SetFrameSize(unsigned width, unsigned height)
   m_height = height;
   PColourConverter::FillYUV420P(0, 0, m_width, m_height, m_width, m_height,
                                 m_frameStore.GetPointer(m_width*m_height*3/2),
-                                0, 0, 0);
+                                m_bgFillRed, m_bgFillGreen, m_bgFillBlue);
 
   m_mutex.Signal();
   return true;
@@ -611,15 +615,23 @@ bool OpalVideoMixer::MixStreams(RTP_DataFrame & frame)
     case eGrid :
       x = left = 0;
       y = 0;
-      switch (m_inputStreams.size()) {
+      if (m_lastStreamCount != m_inputStreams.size()) {
+        PColourConverter::FillYUV420P(0, 0, m_width, m_height, m_width, m_height,
+                                      m_frameStore.GetPointer(),
+                                      m_bgFillRed, m_bgFillGreen, m_bgFillBlue);
+        m_lastStreamCount = m_inputStreams.size();
+      }
+      switch (m_lastStreamCount) {
         case 0 :
         case 1 :
           w = m_width;
           h = m_height;
           break;
-          break;
 
         case 2 :
+          y = m_height/4;
+          // Fall into next case
+
         case 3 :
         case 4 :
           w = m_width/2;
@@ -910,14 +922,28 @@ OpalMediaStream * OpalMixerConnection::CreateMediaStream(const OpalMediaFormat &
 void OpalMixerConnection::OnStartMediaPatch(OpalMediaPatch & patch)
 {
   OpalLocalConnection::OnStartMediaPatch(patch);
-  m_node->UseMediaBypass(patch.GetSource().GetSessionID());
+  m_node->UseMediaPassThrough(patch.GetSource().GetSessionID());
 }
 
 
-void OpalMixerConnection::ApplyStringOptions(OpalConnection::StringOptions & stringOptions)
+void OpalMixerConnection::OnApplyStringOptions()
 {
-  SetListenOnly(stringOptions.GetBoolean(OPAL_OPT_LISTEN_ONLY, GetListenOnly()));
-  OpalLocalConnection::ApplyStringOptions(stringOptions);
+  OpalLocalConnection::OnApplyStringOptions();
+  SetListenOnly(m_stringOptions.GetBoolean(OPAL_OPT_LISTEN_ONLY, GetListenOnly()));
+}
+
+
+bool OpalMixerConnection::SendUserInputString(const PString & value)
+{
+  m_node->BroadcastUserInput(this, value);
+  return true;
+}
+
+
+PBoolean OpalMixerConnection::SendUserInputTone(char tone, unsigned /*duration*/)
+{
+  m_node->BroadcastUserInput(this, tone);
+  return true;
 }
 
 
@@ -950,12 +976,11 @@ OpalMixerMediaStream::OpalMixerMediaStream(OpalConnection & conn,
                                                             bool listenOnly)
   : OpalMediaStream(conn, format, sessionID, isSource)
   , m_node(node)
+  , m_listenOnly(listenOnly)
 #if OPAL_VIDEO
   , m_video(mediaFormat.GetMediaType() == OpalMediaType::Video())
 #endif
 {
-  paused = IsSink() && listenOnly;
-
   /* We are a bit sneaky here. OpalCall::OpenSourceMediaStream will have
      selected the network codec (e.g. G.723.1) anbd passed it to us, but for
      the case of incoming media to the mixer (sink), we switch it to the raw
@@ -994,7 +1019,9 @@ PBoolean OpalMixerMediaStream::Open()
     return false;
   }
 
-  if (!paused && !m_node->AttachStream(this))
+  SetPaused(IsSink() && m_listenOnly);
+
+  if (!IsPaused() && !m_node->AttachStream(this))
     return false;
 
   return OpalMediaStream::Open();
@@ -1014,9 +1041,6 @@ PBoolean OpalMixerMediaStream::Close()
 
 PBoolean OpalMixerMediaStream::WritePacket(RTP_DataFrame & packet)
 {
-  if (paused)
-    return true;
-
 #if OPAL_VIDEO
   if (m_video)
     return m_node->WriteVideo(GetID(), packet);
@@ -1038,6 +1062,26 @@ PBoolean OpalMixerMediaStream::RequiresPatchThread() const
 }
 
 
+bool OpalMixerMediaStream::EnableJitterBuffer(bool enab) const
+{
+  if (IsSource())
+    return false;
+
+  unsigned minJitter, maxJitter;
+
+  if (enab) {
+    minJitter = connection.GetMinAudioJitterDelay();
+    maxJitter = connection.GetMaxAudioJitterDelay();
+  }
+  else {
+    minJitter = 0;
+    maxJitter = 0;
+  }
+
+  return m_node->SetJitterBufferSize(GetID(), minJitter, maxJitter);
+}
+
+
 ///////////////////////////////////////////////////////////////////////////////
 
 OpalMixerNode::OpalMixerNode(OpalMixerNodeManager & manager,
@@ -1148,7 +1192,7 @@ void OpalMixerNode::AttachConnection(OpalConnection * connection)
 
   m_connections.Append(connection);
 
-  UseMediaBypass(0);
+  UseMediaPassThrough(0);
 }
 
 
@@ -1158,7 +1202,7 @@ void OpalMixerNode::DetachConnection(OpalConnection * connection)
     return;
 
   if (m_connections.Remove(connection))
-    UseMediaBypass(0, connection);
+    UseMediaPassThrough(0, connection);
 }
 
 
@@ -1180,10 +1224,6 @@ bool OpalMixerNode::AttachStream(OpalMixerMediaStream * stream)
   if (stream->IsSink())
     return m_audioMixer.AddStream(stream->GetID());
 
-  OpalConnection & connection = stream->GetConnection();
-  m_audioMixer.SetJitterBufferSize(stream->GetID(),
-                                   connection.GetMinAudioJitterDelay(),
-                                   connection.GetMaxAudioJitterDelay());
   m_audioMixer.m_outputStreams.Append(stream);
   return true;
 }
@@ -1212,9 +1252,9 @@ void OpalMixerNode::DetachStream(OpalMixerMediaStream * stream)
 }
 
 
-void OpalMixerNode::UseMediaBypass(unsigned sessionID, OpalConnection * connection)
+void OpalMixerNode::UseMediaPassThrough(unsigned sessionID, OpalConnection * connection)
 {
-  if (m_info->m_noMediaBypass)
+  if (!m_info->m_mediaPassThru)
     return;
 
   PSafePtr<OpalConnection> other2;
@@ -1242,7 +1282,16 @@ void OpalMixerNode::UseMediaBypass(unsigned sessionID, OpalConnection * connecti
   if (other1 == NULL)
     return;
 
-  OpalManager::SetMediaBypass(*other1, *other2, m_connections.GetSize() == 2, sessionID);
+  OpalManager::SetMediaPassThrough(*other1, *other2, m_connections.GetSize() == 2, sessionID);
+}
+
+
+void OpalMixerNode::BroadcastUserInput(const OpalConnection * connection, const PString & value)
+{
+  for (PSafePtr<OpalConnection> conn(m_connections, PSafeReference); conn != NULL; ++conn) {
+    if (connection != conn)
+      conn->OnUserInputString(value);
+  }
 }
 
 
@@ -1430,15 +1479,22 @@ OpalMixerNode::VideoMixer::~VideoMixer()
 
 bool OpalMixerNode::VideoMixer::OnMixed(RTP_DataFrame * & output)
 {
-  std::map<OpalMediaFormat, RTP_DataFrameList> cachedVideo;
+  std::map<PString, RTP_DataFrameList> cachedVideo;
+  std::map<PString, RTP_DataFrame>     cachedFrameStore;
 
   for (PSafePtr<OpalMixerMediaStream> stream(m_outputStreams, PSafeReadOnly); stream != NULL; ++stream) {
     OpalMediaFormat mediaFormat = stream->GetMediaFormat();
     if (mediaFormat == OpalYUV420P)
       stream->PushPacket(*output);
     else {
-      if (cachedVideo.find(mediaFormat) == cachedVideo.end()) {
-        OpalTranscoder * transcoder = m_transcoders.GetAt(mediaFormat);
+      unsigned width = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameWidthOption());
+      unsigned height = mediaFormat.GetOptionInteger(OpalVideoFormat::FrameHeightOption());
+
+      PStringStream key;
+      key << mediaFormat << width << height;
+
+      if (cachedVideo.find(key) == cachedVideo.end()) {
+        OpalTranscoder * transcoder = m_transcoders.GetAt(key);
         if (transcoder == NULL) {
           transcoder = OpalTranscoder::Create(OpalYUV420P, mediaFormat);
           if (transcoder == NULL) {
@@ -1447,9 +1503,32 @@ bool OpalMixerNode::VideoMixer::OnMixed(RTP_DataFrame * & output)
             stream->Close();
             continue;
           }
-          m_transcoders.SetAt(mediaFormat, transcoder);
+          PTRACE(3, "MixerNode\tCreated transcoder to "
+                 << mediaFormat << " for stream id " << stream->GetID());
+          m_transcoders.SetAt(key, transcoder);
         }
-        if (!transcoder->ConvertFrames(*output, cachedVideo[mediaFormat])) {
+
+        const RTP_DataFrame * rawRTP;
+        const OpalVideoTranscoder::FrameHeader * header = (const OpalVideoTranscoder::FrameHeader *)output->GetPayloadPtr();
+        if (header->width == width && header->height == height)
+          rawRTP = output;
+        else {
+          if (cachedFrameStore.find(key) == cachedFrameStore.end()) {
+            RTP_DataFrame & store = cachedFrameStore[key];
+            store.SetPayloadSize(width*height*3/2+sizeof(OpalVideoTranscoder::FrameHeader));
+            OpalVideoTranscoder::FrameHeader * resized = (OpalVideoTranscoder::FrameHeader *)store.GetPayloadPtr();
+            resized->width = width;
+            resized->height = height;
+            PColourConverter::CopyYUV420P(0, 0, header->width, header->height,
+                                            header->width, header->height, OPAL_VIDEO_FRAME_DATA_PTR(header),
+                                          0, 0, width, height,
+                                          width, height, OPAL_VIDEO_FRAME_DATA_PTR(resized),
+                                          PVideoFrameInfo::eScale);
+          }
+          rawRTP = &cachedFrameStore[key];
+        }
+
+        if (!transcoder->ConvertFrames(*rawRTP, cachedVideo[key])) {
           PTRACE(2, "MixerNode\tCould not convert video to "
                  << mediaFormat << " for stream id " << stream->GetID());
           stream->Close();
@@ -1457,7 +1536,7 @@ bool OpalMixerNode::VideoMixer::OnMixed(RTP_DataFrame * & output)
         }
       }
 
-      RTP_DataFrameList & list = cachedVideo[mediaFormat];
+      RTP_DataFrameList & list = cachedVideo[key];
       for (RTP_DataFrameList::iterator frame = list.begin(); frame != list.end(); ++frame)
         stream->PushPacket(*frame);
     }
