@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 24687 $
+ * $Revision: 26088 $
  * $Author: rjongbloed $
- * $Date: 2010-09-12 23:10:00 -0500 (Sun, 12 Sep 2010) $
+ * $Date: 2011-06-28 01:51:16 -0500 (Tue, 28 Jun 2011) $
  */
 
 #include <ptlib.h>
@@ -190,6 +190,7 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
     source.SetDataSize(packetSize, packetTime);
     sinkStream->SetDataSize(packetSize, packetTime);
     PTRACE(3, "Patch\tAdded direct media stream sink " << *sinkStream);
+    EnableJitterBuffer();
     return true;
   }
 
@@ -210,6 +211,7 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
            << " using transcoder " << *sink->primaryCodec << ", data size=" << sinkStream->GetDataSize());
   }
   else {
+    PTRACE(4, "Patch\tCreating two stage transcoders for " << sourceFormat << "->" << destinationFormat << " with ID " << id);
     OpalMediaFormat intermediateFormat;
     if (!OpalTranscoder::FindIntermediateFormat(sourceFormat, destinationFormat,
                                                 intermediateFormat)) {
@@ -236,8 +238,10 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
 
     sink->primaryCodec = OpalTranscoder::Create(sourceFormat, intermediateFormat, (const BYTE *)id, id.GetLength());
     sink->secondaryCodec = OpalTranscoder::Create(intermediateFormat, destinationFormat, (const BYTE *)id, id.GetLength());
+    if (sink->primaryCodec == NULL || sink->secondaryCodec == NULL)
+      return false;
 
-    PTRACE(4, "Patch\tCreated two stage codec " << sourceFormat << "/" << intermediateFormat << "/" << destinationFormat << " with ID " << id);
+    PTRACE(3, "Patch\tCreated two stage codec " << sourceFormat << "/" << intermediateFormat << "/" << destinationFormat << " with ID " << id);
 
     sink->primaryCodec->SetMaxOutputSize(sink->secondaryCodec->GetOptimalDataFrameSize(true));
     sink->primaryCodec->SetSessionID(source.GetSessionID());
@@ -256,6 +260,7 @@ PBoolean OpalMediaPatch::AddSink(const OpalMediaStreamPtr & sinkStream)
   }
 
   source.SetDataSize(sink->primaryCodec->GetOptimalDataFrameSize(true), destinationFormat.GetFrameTime());
+  EnableJitterBuffer();
   return true;
 }
 
@@ -267,6 +272,8 @@ void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
 
   PTRACE(3, "Patch\tRemoving media stream sink " << *stream);
 
+  bool closeSource = false;
+
   inUse.StartWrite();
 
   for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
@@ -277,10 +284,16 @@ void OpalMediaPatch::RemoveSink(const OpalMediaStreamPtr & stream)
     }
   }
 
-  if (m_bypassFromPatch != NULL && sinks.IsEmpty())
-    m_bypassFromPatch->SetBypassPatch(NULL);
+  if (sinks.IsEmpty()) {
+    closeSource = true;
+    if (m_bypassFromPatch != NULL)
+      m_bypassFromPatch->SetBypassPatch(NULL);
+  }
 
   inUse.EndWrite();
+
+  if (closeSource  && source.GetPatch() == this)
+    source.Close();
 }
 
 
@@ -421,7 +434,7 @@ PBoolean OpalMediaPatch::RemoveFilter(const PNotifier & filter, const OpalMediaF
     }
   }
 
-  PTRACE(2, "OpalCon\tNo filter to remove for stage " << stage);
+  PTRACE(3, "OpalCon\tNo filter to remove for stage " << stage);
   return false;
 }
 
@@ -461,9 +474,14 @@ PBoolean OpalMediaPatch::ExecuteCommand(const OpalMediaCommand & command, PBoole
   if (fromSink)
     return source.ExecuteCommand(command);
 
-  PBoolean atLeastOne = PFalse;
-  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s)
-    atLeastOne = s->ExecuteCommand(command) || atLeastOne;
+  bool atLeastOne = false;
+
+  PList<Sink> & sinksToUse = m_bypassActive && m_bypassToPatch != NULL ? m_bypassToPatch->sinks : sinks;
+
+  for (PList<Sink>::iterator s = sinksToUse.begin(); s != sinksToUse.end(); ++s) {
+    if (s->ExecuteCommand(command))
+      atLeastOne = true;
+  }
 
   return atLeastOne;
 }
@@ -482,14 +500,21 @@ void OpalMediaPatch::SetCommandNotifier(const PNotifier & notifier, PBoolean fro
 }
 
 
-void OpalMediaPatch::SetPaused(bool pause)
+bool OpalMediaPatch::SetPaused(bool pause)
 {
   PReadWaitAndSignal mutex(inUse);
 
-  source.SetPaused(pause);
+  bool atLeastOne = source.SetPaused(pause, true);
 
-  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s)
-    s->stream->SetPaused(pause);
+  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
+    if (s->stream->SetPaused(pause, true))
+      atLeastOne = true;
+  }
+
+  if (!pause)
+    Start();
+
+  return atLeastOne;
 }
 
 
@@ -500,15 +525,29 @@ bool OpalMediaPatch::OnStartMediaPatch()
   if (source.IsSynchronous())
     return false;
 
+  return EnableJitterBuffer();
+}
+
+
+bool OpalMediaPatch::EnableJitterBuffer()
+{
   PReadWaitAndSignal mutex(inUse);
 
-  for (PList<Sink>::iterator s = sinks.begin(); s != sinks.end(); ++s) {
-    if (s->stream->IsSynchronous()) {
-      source.EnableJitterBuffer();
+  bool enab = m_bypassToPatch == NULL;
+
+  PList<Sink>::iterator s;
+  for (s = sinks.begin(); s != sinks.end(); ++s) {
+    if (s->stream->EnableJitterBuffer(enab)) {
+      source.EnableJitterBuffer(false);
       return false;
     }
   }
-	
+
+  for (s = sinks.begin(); s != sinks.end(); ++s) {
+    if (s->stream->IsSynchronous() && source.EnableJitterBuffer(enab))
+      return false;
+  }
+
   return true;
 }
 
@@ -518,6 +557,8 @@ void OpalMediaPatch::Main()
   PTRACE(4, "Patch\tThread started for " << *this);
 	
   bool asynchronous = OnStartMediaPatch();
+  PAdaptiveDelay asynchPacing;
+  PThread::Times lastThreadTimes;
   PTimeInterval lastTick;
 
   /* Note the RTP frame is outside loop so that a) it is more efficient
@@ -530,6 +571,12 @@ void OpalMediaPatch::Main()
   RTP_DataFrame sourceFrame(0);
 
   while (source.IsOpen()) {
+    if (source.IsPaused()) {
+      PThread::Sleep(100);
+      continue;
+    }
+
+    sourceFrame.MakeUnique();
     sourceFrame.SetPayloadType(source.GetMediaFormat().GetPayloadType());
 
     // We do the following to make sure that the buffer size is large enough,
@@ -551,16 +598,34 @@ void OpalMediaPatch::Main()
       break;
     }
  
+    if (asynchronous)
+      asynchPacing.Delay(10);
+
     /* Don't starve the CPU if we have idle frames and the no source or
        destination is synchronous. Note that performing a Yield is not good
        enough, as the media patch threads are high priority and will consume
        all available CPU if allowed. Also just doing a sleep each time around
        the loop slows down video where you get clusters of packets thrown at
        us, want to clear them as quickly as possible out of the UDP OS buffers
-       or we overflow and lose some. Best compromise is to every 100ms, sleep
-       for 10ms so can not use more than about 90% of CPU. */
-    if (asynchronous && PTimer::Tick() - lastTick > 100) {
-      PThread::Sleep(10);
+       or we overflow and lose some. Best compromise is to every X ms, sleep
+       for X/10 ms so can not use more than about 90% of CPU. */
+#if P_CONFIG_FILE
+    static const unsigned SampleTimeMS = PConfig(PConfig::Environment).GetInteger("OPAL_MEDIA_PATCH_CPU_CHECK", 1000);
+#else
+    static const unsigned SampleTimeMS = 1000;
+#endif
+    static const unsigned ThresholdPercent = 90;
+    if (PTimer::Tick() - lastTick > SampleTimeMS) {
+      PThread::Times threadTimes;
+      if (PThread::Current()->GetTimes(threadTimes)) {
+        PTRACE(5, "Patch\tCPU for " << *this << " is " << threadTimes);
+        if ((threadTimes.m_user - lastThreadTimes.m_user + threadTimes.m_kernel - lastThreadTimes.m_kernel)
+                                      > (threadTimes.m_real - lastThreadTimes.m_real)*ThresholdPercent/100) {
+          PTRACE(2, "Patch\tGreater that 90% CPU usage for " << *this);
+          PThread::Sleep(SampleTimeMS*(100-ThresholdPercent)/100);
+        }
+        lastThreadTimes = threadTimes;
+      }
       lastTick = PTimer::Tick();
     }
   }
@@ -573,8 +638,6 @@ void OpalMediaPatch::Main()
 
 bool OpalMediaPatch::SetBypassPatch(OpalMediaPatch * patch)
 {
-  PTRACE(4, "Patch\tSetting media patch bypass to " << patch << " on " << *this);
-
   PWriteWaitAndSignal mutex(inUse);
 
   if (!PAssert(m_bypassFromPatch == NULL, PLogicError))
@@ -583,6 +646,8 @@ bool OpalMediaPatch::SetBypassPatch(OpalMediaPatch * patch)
   if (m_bypassToPatch == patch)
     return true; // Already set
 
+  PTRACE(4, "Patch\tSetting media patch bypass to " << patch << " on " << *this);
+
   if (m_bypassToPatch != NULL) {
     if (!PAssert(m_bypassToPatch->m_bypassFromPatch == this, PLogicError))
       return false;
@@ -601,8 +666,14 @@ bool OpalMediaPatch::SetBypassPatch(OpalMediaPatch * patch)
   m_bypassToPatch = patch;
 
 #if OPAL_VIDEO
+  if (m_videoDecoder)
+    source.ExecuteCommand(OpalVideoUpdatePicture());
+  else
+    EnableJitterBuffer();
+
   m_bypassActive = m_bypassToPatch != NULL && !m_videoDecoder;
 #else
+  EnableJitterBuffer();
   m_bypassActive = m_bypassToPatch != NULL;
 #endif
 
@@ -705,6 +776,11 @@ bool OpalMediaPatch::Sink::CannotTranscodeFrame(OpalTranscoder & codec, RTP_Data
 {
   RTP_DataFrame::PayloadTypes pt = frame.GetPayloadType();
 
+  if (!codec.AcceptEmptyPayload() && frame.GetPayloadSize() == 0) {
+    frame.SetPayloadType(codec.GetPayloadType(false));
+    return true;
+  }
+
   if (!codec.AcceptComfortNoise()) {
     if (pt == RTP_DataFrame::CN || pt == RTP_DataFrame::Cisco_CN) {
       PTRACE(4, "Patch\tRemoving comfort noise frame with payload type " << pt);
@@ -733,11 +809,6 @@ bool OpalMediaPatch::Sink::CannotTranscodeFrame(OpalTranscoder & codec, RTP_Data
     return true;
   }
 
-  if (!codec.AcceptEmptyPayload() && frame.GetPayloadSize() == 0) {
-    frame.SetPayloadType(codec.GetPayloadType(false));
-    return true;
-  }
-
   return false;
 }
 
@@ -791,7 +862,7 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
     bool forceIFrame = false;
     bool s = RateControlExceeded(forceIFrame);
     if (forceIFrame)
-      stream->ExecuteCommand(OpalVideoUpdatePicture2(sourceFrame.GetSequenceNumber(), sourceFrame.GetTimestamp()));
+      stream->ExecuteCommand(OpalVideoUpdatePicture());
     if (s) {
       if (secondaryCodec == NULL) {
         bool wasIFrame = false;
@@ -840,7 +911,7 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
         patch.FilterFrame(*interFrame, primaryCodec->GetOutputFormat());
         if (!stream->WritePacket(*interFrame))
           return (writeSuccessful = false);
-        sourceFrame.SetTimestamp(interFrame->GetTimestamp());
+        primaryCodec->CopyTimestamp(sourceFrame, *interFrame, false);
         continue;
       }
       intermediateFrames.RemoveAll();
@@ -854,7 +925,7 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
     if (secondaryCodec == NULL) {
       if (!stream->WritePacket(*interFrame))
         return (writeSuccessful = false);
-      sourceFrame.SetTimestamp(interFrame->GetTimestamp());
+      primaryCodec->CopyTimestamp(sourceFrame, *interFrame, false);
       continue;
     }
 
@@ -873,7 +944,7 @@ bool OpalMediaPatch::Sink::WriteFrame(RTP_DataFrame & sourceFrame)
       patch.FilterFrame(*finalFrame, secondaryCodec->GetOutputFormat());
       if (!stream->WritePacket(*finalFrame))
         return (writeSuccessful = false);
-      sourceFrame.SetTimestamp(finalFrame->GetTimestamp());
+      secondaryCodec->CopyTimestamp(sourceFrame, *finalFrame, false);
     }
   }
 
