@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25734 $
+ * $Revision: 26155 $
  * $Author: rjongbloed $
- * $Date: 2011-05-10 21:41:47 -0500 (Tue, 10 May 2011) $
+ * $Date: 2011-07-10 21:14:36 -0500 (Sun, 10 Jul 2011) $
  */
 
 #include <ptlib.h>
@@ -202,6 +202,10 @@ PString SDPMediaFormat::GetFMTP() const
   for (PINDEX i = 0; i < mediaFormat.GetOptionCount(); i++) {
     const OpalMediaOption & option = mediaFormat.GetOption(i);
     const PString & name = option.GetFMTPName();
+    // If the FMTP name for option is "FMTP" then it is the WHOLE thing
+    // any other options with FMTP names are ignored.
+    if (name == "FMTP")
+      return option.AsString();
     if (!name.IsEmpty() && !used.Contains(name)) {
       used.Include(name);
       PString value = option.AsString();
@@ -266,9 +270,11 @@ void SDPMediaFormat::SetMediaFormatOptions(OpalMediaFormat & mediaFormat) const
   // Fill in the default values for (possibly) missing FMTP options
   for (PINDEX i = 0; i < mediaFormat.GetOptionCount(); i++) {
     OpalMediaOption & option = const_cast<OpalMediaOption &>(mediaFormat.GetOption(i));
-    if (!option.GetFMTPName().IsEmpty() && !option.GetFMTPDefault().IsEmpty())
-      option.FromString(option.GetFMTPDefault());
+    PString value = option.GetFMTPDefault();
+    if (!value.IsEmpty())
+      option.FromString(value);
   }
+
   // Set the frame size options from media's ptime & maxptime attributes
   if (m_parent.GetPTime() >= SDP_MIN_PTIME && mediaFormat.HasOption(OpalAudioFormat::TxFramesPerPacketOption())) {
     unsigned frames = (m_parent.GetPTime() * mediaFormat.GetTimeUnits()) / mediaFormat.GetFrameTime();
@@ -276,6 +282,7 @@ void SDPMediaFormat::SetMediaFormatOptions(OpalMediaFormat & mediaFormat) const
       frames = 1;
     mediaFormat.SetOptionInteger(OpalAudioFormat::TxFramesPerPacketOption(), frames);
   }
+
   if (m_parent.GetMaxPTime() >= SDP_MIN_PTIME && mediaFormat.HasOption(OpalAudioFormat::RxFramesPerPacketOption())) {
     unsigned frames = (m_parent.GetMaxPTime() * mediaFormat.GetTimeUnits()) / mediaFormat.GetFrameTime();
     if (frames < 1)
@@ -287,6 +294,15 @@ void SDPMediaFormat::SetMediaFormatOptions(OpalMediaFormat & mediaFormat) const
   if (mediaFormat.SetOptionString("FMTP", m_fmtp))
     return;
 
+  // Look for a specific option with "FMTP" as it's name, it is the whole thing then
+  for (PINDEX i = 0; i < mediaFormat.GetOptionCount(); i++) {
+    const OpalMediaOption & option = mediaFormat.GetOption(i);
+    if (option.GetFMTPName() == "FMTP") {
+      mediaFormat.SetOptionValue(option.GetName(), m_fmtp);
+      return;
+    }
+  }
+
   // Save the RTCP feedback (RFC4585) capability.
   if (!m_rtcp_fb.IsEmpty())
     mediaFormat.AddOption(new OpalMediaOptionString("RTCP-FB", false, m_rtcp_fb), true);
@@ -1487,7 +1503,7 @@ bool SDPSessionDescription::Decode(const PString & str, const OpalMediaFormatLis
   }
 
   if (currentMedia != NULL) {
-    PTRACE(3, "SDP\tParsed media session with " << currentMedia->GetSDPMediaFormats().GetSize()
+    PTRACE(3, "SDP\tParsed final media session with " << currentMedia->GetSDPMediaFormats().GetSize()
                                                 << " '" << currentMedia->GetSDPMediaType() << "' formats");
     if (!currentMedia->PostDecode(mediaFormats))
       ok = false;
