@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision: 24092 $
+ * $Revision: 26155 $
  * $Author: rjongbloed $
- * $Date: 2010-03-04 00:22:29 -0600 (Thu, 04 Mar 2010) $
+ * $Date: 2011-07-10 21:14:36 -0500 (Sun, 10 Jul 2011) $
  */
 
 /*
@@ -85,6 +85,12 @@ PCaselessString SDPFaxMediaDescription::GetSDPTransportType() const
   return "udptl";
 }
 
+SDPMediaDescription * SDPFaxMediaDescription::CreateEmpty() const
+{
+  return new SDPFaxMediaDescription(OpalTransportAddress());
+}
+
+
 PString SDPFaxMediaDescription::GetSDPMediaType() const 
 { 
   return "image"; 
@@ -144,13 +150,38 @@ void SDPFaxMediaDescription::ProcessMediaOptions(SDPMediaFormat & /*sdpFormat*/,
   if (mediaFormat.GetMediaType() == OpalMediaType::Fax()) {
     for (PINDEX i = 0; i < mediaFormat.GetOptionCount(); ++i) {
       const OpalMediaOption & option = mediaFormat.GetOption(i);
-      if (option.GetName().Left(3) *= "t38") 
-        t38Attributes.SetAt(option.GetName(), option.AsString());
+      if (option.GetName().Left(3) *= "t38") {
+        PString value = option.AsString();
+        if (value != option.GetFMTPDefault())
+          t38Attributes.SetAt(option.GetName(), option.AsString());
+      }
     }
   }
 }
 
-/////////////////////////////////////////////////////////////////////////////
+
+bool SDPFaxMediaDescription::PostDecode(const OpalMediaFormatList & mediaFormats)
+{
+  if (!SDPMediaDescription::PostDecode(mediaFormats))
+    return false;
+
+  for (SDPMediaFormatList::iterator format = formats.begin(); format != formats.end(); ++format) {
+    OpalMediaFormat & mediaFormat = format->GetWritableMediaFormat();
+    if (mediaFormat.GetMediaType() == OpalMediaType::Fax()) {
+      for (PINDEX i = 0; i < t38Attributes.GetSize(); ++i) {
+        PString key = t38Attributes.GetKeyAt(i);
+        PString data = t38Attributes.GetDataAt(i);
+        if (!mediaFormat.SetOptionValue(key, data)) {
+          PTRACE(2, "T38\tCould not set option \"" << key << "\" to \"" << data << '"');
+        }
+      }
+      PTRACE(5, "T38\tMedia format set from SDP:\n" << setw(-1) << mediaFormat);
+    }
+  }
+
+  return true;
+}
+
 
 #endif // OPAL_T38_CAPABILITY
 #endif // OPAL_SIP
